from __future__ import annotations

from tcsoa.gen.Manufacturing._2012_09.DataManagement import CreateConfigInput, CreateConfigResponse, ApplyConfigInput
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService
from tcsoa.gen.Manufacturing._2012_09.Validation import ValidationsChecksObjectsResponse, ValidationCheckExecutionParam, ValidationsChecksExecutionResponse


class DataManagementService(TcService):

    @classmethod
    def createOrUpdateConfigObjects(cls, input: List[CreateConfigInput]) -> CreateConfigResponse:
        """
        Creates or updates the configuration objects based on the input data.
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateConfigObjects',
            library='Manufacturing',
            service_date='2012_09',
            service_name='DataManagement',
            params={'input': input},
            response_cls=CreateConfigResponse,
        )

    @classmethod
    def applyConfigObjects(cls, input: List[ApplyConfigInput]) -> ServiceData:
        """
        Apply configuration objects to applicable business objects.
        """
        return cls.execute_soa_method(
            method_name='applyConfigObjects',
            library='Manufacturing',
            service_date='2012_09',
            service_name='DataManagement',
            params={'input': input},
            response_cls=ServiceData,
        )


class ValidationService(TcService):

    @classmethod
    def executeValidations(cls, input: List[ValidationCheckExecutionParam], failAllOnError: bool) -> ValidationsChecksExecutionResponse:
        """
        This SOA function is to execute the validation checks by the user choice from the UI
        """
        return cls.execute_soa_method(
            method_name='executeValidations',
            library='Manufacturing',
            service_date='2012_09',
            service_name='Validation',
            params={'input': input, 'failAllOnError': failAllOnError},
            response_cls=ValidationsChecksExecutionResponse,
        )

    @classmethod
    def getAllValidations(cls) -> ValidationsChecksObjectsResponse:
        """
        This SOA function is to get all the customized registered callback to show the user in the UI
        """
        return cls.execute_soa_method(
            method_name='getAllValidations',
            library='Manufacturing',
            service_date='2012_09',
            service_name='Validation',
            params={},
            response_cls=ValidationsChecksObjectsResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetACLInfoResponse(TcBaseObj):
    """
    This structure holds the list of effective ACL reports for each of the given business objects.  Each effective ACL
    report contains the business object and corresponding effective ACL information.
    
    :var aclInfoList: List of derived Effective ACL reports for each of the given business objects.
    :var serviceData: Object that holds the partial errors that occurred while getting the effective ACL information
    for any of the given business objects.
    """
    aclInfoList: List[ACLInfos] = ()
    serviceData: ServiceData = None


@dataclass
class GetProtectionReportResponse(TcBaseObj):
    """
    This structure contains list of extra protection reports one for of each of the given business objects for the
    given user.
    
    :var protectionReports: List of extra protection report objects one for each of the given business objects.
    :var serviceData: Object that holds the partial errors occurred while deriving extra protection  information of any
    of the given business objects.
    """
    protectionReports: List[ProtectionReport] = ()
    serviceData: ServiceData = None


@dataclass
class Privilege(TcBaseObj):
    """
    This structure holds the name of the given privilege and its verdict that indicates if the privilege is granted or
    denied.
    
    :var privilegeName: The name of the access privilege.
    :var verdict: Value true means the privilege is granted and false means the privilege is denied.
    """
    privilegeName: str = ''
    verdict: bool = False


@dataclass
class PrivilegeReport(TcBaseObj):
    """
    This structure holds the business object for which access privileges are successfully evaluated and corresponding
    privileges information. Privilege information includes the privilege name and corresponding verdict for each of the
    given privileges.
    
    :var object: Business object on which access privileges are evaluated successfully  for the given groupMember.
    :var privilegeInfos: A list of evaluated access privileges and corresponding verdicts for the given groupMember on
    the business object.
    """
    object: BusinessObject = None
    privilegeInfos: List[Privilege] = ()


@dataclass
class PrivilegeSettingInput(TcBaseObj):
    """
    This structure holds the object on which access privileges need to be modified and the sets of granted, denied and
    unset privileges for the given accessor type.
    
    :var object: POM_application_object or a named ACL.
    :var grant: A list of access privilege names that need to be granted.
    :var revoke: A list of access privilege names that need to be denied.
    :var unset: A list of access privilege names that need to be unset.
    """
    object: BusinessObject = None
    grant: List[str] = ()
    revoke: List[str] = ()
    unset: List[str] = ()


@dataclass
class Protection(TcBaseObj):
    """
    This structure contains the derived extra protection information corresponding to a single access privilege on a
    given business object for the given user.
    
    :var privilege: The name of the access privilege.
    :var verdict: True or false. True means the privilege is granted and false means the privilege is denied.
    :var rules: A list of access rules evalued to arrive at the verdict for the privilege.
    :var ruleEvaluation: A list of access rule arrangements involved in arriving at the verdict.
    :var acl: The name of the named ACL that is used to determine the verdict for this privilege. For object ACLs the
    name is "OBJECT".
    :var accessorType: The accessor type name corresponding to the ACE that involved in arriving at the privilege
    verdict.
    :var accessorId: The name of the accessor that is used in the ACE that is used to determine the privilege.
    """
    privilege: str = ''
    verdict: bool = False
    rules: List[str] = ()
    ruleEvaluation: List[str] = ()
    acl: str = ''
    accessorType: str = ''
    accessorId: str = ''


@dataclass
class ProtectionReport(TcBaseObj):
    """
    This structure holds the business object and corresponding derived extra protection information for all the access
    privileges for the given user.
    
    :var object: POM_object on which extra protection information is derived for the given user.
    :var protections: The derived extra protection information corresponding to the single business object.
    """
    object: BusinessObject = None
    protections: List[Protection] = ()


@dataclass
class CheckAccessorPrivilegesResponse(TcBaseObj):
    """
    A list of privilegeReport objects for each of the given business objects and each given privilege. Each
    PrivilegeReport object contains the business object and verdicts for all the given privileges.
    
    :var privilegeReports: A list of PrivilegeReport objects one for each of the given business object.
    :var serviceData: Object that holds the partial errors that resulted while determining the privileges on a business
    object through access rules evaluation.
    """
    privilegeReports: List[PrivilegeReport] = ()
    serviceData: ServiceData = None


@dataclass
class ACLInfo(TcBaseObj):
    """
    Structure to hold the ACE information for a single row in the effective ACL table like granted privileges, revoked
    privileges, Accessor Type name, Accessor Id and the ACL name.
    
    :var grantedNames: Names access privilege that are granted to the user on a given object.
    :var revokedNames: Names access privilege that are revoked to the user on a given object.
    :var accessorType: The name of the accessor type in the ACE that is applicable to the given business object and to
    the user.
    :var accessorId: The name of the accessor id in the ACE that is applicable to the given business object and to the
    user.
    :var namedACL: The name of the name of the ACL in the ACE that is applicable to the given business object and to
    the user.  Name for the object ACL is "ORBJECT".
    """
    grantedNames: List[str] = ()
    revokedNames: List[str] = ()
    accessorType: str = ''
    accessorId: str = ''
    namedACL: str = ''


@dataclass
class ACLInfos(TcBaseObj):
    """
    This structure holds the business object and corresponding effective ACL information derived through access rule
    evaluation.  Each effective ACL contains a list of ACE objects. Each ACE object is a single row in the effective
    ACL table that contains information like granted privileges, revoked privileges, Accessor Type name, Accessor Id
    and the ACL name.
    
    :var object: The business object for which effective ACL was derived.
    :var aclInfos: List of ACE objects returned from the server that make up the effective ACL for the business object.
    """
    object: BusinessObject = None
    aclInfos: List[ACLInfo] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Awb0ProductContextInfo, Awb0SavedBookmark
from typing import List
from tcsoa.gen.ActiveWorkspaceBom._2015_10.OccurrenceManagementCad import SavedBookmarksBOMWindowsResponse, ProductContextBOMWindowResponse
from tcsoa.base import TcService


class OccurrenceManagementCadService(TcService):

    @classmethod
    def createBOMWindowsFromBookmarks(cls, savedBookmarks: List[Awb0SavedBookmark], expandToSelection: bool) -> SavedBookmarksBOMWindowsResponse:
        """
        This operation creates BOM windows (BOMWindow objects) from Active Workspace saved bookmarks (Awb0SavedBookmark
        objects).  A BOM window will be created for each product contained within each saved bookmark.  With each
        BOMwindow there will be a corresponding list of BOMLline objects that identify any selections included in the
        saved bookmark.  The product of the Awb0ProductContextInfo that is associated with each saved bookmark will be
        set as the top line of the BOMWindow.  The product contexts also contain information that will be set on the
        BOM window when it is created.  This information may include:
        
        - Revision rule
        - End item
        - Effective date
        - Effective unit
        - Saved Variant rule
        
        
        
        Use cases:
        The user has found a saved bookmark in the Active Context Experience.  The user wants to open the contents of
        the bookmark in the CAD application.  The Active Workspace Client communicates saved bookmark details to the
        CAD application, which then uses it as an input to this operation to create BOM windows.
        
        When BOMWindow objects are needed but only Awb0SavedBookmark objects are accessible, this operation will use
        the necessary information from the Awb0SavedBookmark objects to provide corresponding BOMWindow objects.
        """
        return cls.execute_soa_method(
            method_name='createBOMWindowsFromBookmarks',
            library='ActiveWorkspaceBom',
            service_date='2015_10',
            service_name='OccurrenceManagementCad',
            params={'savedBookmarks': savedBookmarks, 'expandToSelection': expandToSelection},
            response_cls=SavedBookmarksBOMWindowsResponse,
        )

    @classmethod
    def createBOMWindowsFromContexts(cls, productContextInfos: List[Awb0ProductContextInfo], expandToSelection: bool) -> ProductContextBOMWindowResponse:
        """
        This operation creates BOM windows (BOMWindow objects) from Active Workspace product contexts
        (Awb0ProductContextInfo objects).  One BOM window will be created for each product context.  The product of the
        Awb0ProductContextInfo will be set as the top line of the BOMWindow.  With each BOMWindow there will be a
        corresponding list of BOMLine objects that identify any selections included in the product context information.
         The product context contains configuration data that will be set on the BOM Window when it is created.  This
        information may include:
        
        - Revision rule
        - End item
        - Effective date
        - Effective unit
        - Saved Variant rule
        
        
        
        Use cases:
        The user opens an Item-BVR assembly in the Active Content Experience and makes changes to the RevisionRule or
        other configuration parameters.  Then, the user opens it in the CAD application.  The Active Workspace client
        communicates the Awb0ProductContextInfo to the CAD application, which then uses it as an input to this
        operation to create the BOMWindow.
        
        When BOMWindow objects are needed but only Awb0ProductContextInfo objects are accessible, this operation will
        use the necessary information from the Awb0ProductContextInfo objects to create corresponding BOMWindow objects.
        """
        return cls.execute_soa_method(
            method_name='createBOMWindowsFromContexts',
            library='ActiveWorkspaceBom',
            service_date='2015_10',
            service_name='OccurrenceManagementCad',
            params={'productContextInfos': productContextInfos, 'expandToSelection': expandToSelection},
            response_cls=ProductContextBOMWindowResponse,
        )



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService
from tcsoa.gen.BusinessObjects import TimeSheetEntry


class ScheduleManagementService(TcService):

    @classmethod
    def submitTimesheetEntries(cls, timesheetEntries: List[TimeSheetEntry], runInBackground: bool) -> ServiceData:
        """
        This operation submits the list of TimeSheetEntry to workflow. The preference SM_TIMESHEET_APPROVE_WORKFLOW
        will be used to determine the workflow template. If not specified, TimeSheetApproval template will be used.
        """
        return cls.execute_soa_method(
            method_name='submitTimesheetEntries',
            library='ProjectManagement',
            service_date='2018_11',
            service_name='ScheduleManagement',
            params={'timesheetEntries': timesheetEntries, 'runInBackground': runInBackground},
            response_cls=ServiceData,
        )

    @classmethod
    def submitTimesheetEntriesAsync(cls, timesheetEntries: List[TimeSheetEntry]) -> None:
        """
        This operation submits the list of TimeSheetEntry to workflow. The preference SM_TIMESHEET_APPROVE_WORKFLOW
        will be used to determine the workflow template. If not specified, TimeSheetApproval template will be used.
        This operation runs asynchronously in its own server in the background.
        """
        return cls.execute_soa_method(
            method_name='submitTimesheetEntriesAsync',
            library='ProjectManagement',
            service_date='2018_11',
            service_name='ScheduleManagement',
            params={'timesheetEntries': timesheetEntries},
            response_cls=None,
        )



from __future__ import annotations

from tcsoa.gen.Internal.Configurator._2015_03.ConfiguratorManagement import AvailableProductVariabilityInput, AvailableProductVariabilityOutput
from tcsoa.base import TcService


class ConfiguratorManagementService(TcService):

    @classmethod
    def getAvailableProductVariability(cls, input: AvailableProductVariabilityInput) -> AvailableProductVariabilityOutput:
        """
        This operation computes the available features within a set of requested families. The parameters
        criteriaExpression and familiesToTest are used to pass in the input expression and families of interest
        requesting the available features. The operation in response returns the available product variability for the
        requested families satisfying the input expression in criteriaExpression and the constraint(s) present in the
        system. If the input parameter criteriaExpression is empty, then the response will return the product
        variability for the requested families satisfying all the system constraints. If the requested family list is
        empty, the operation will not return any variability information.
        
        Use cases:
        Consider the following Variability data in a Configurator context say MyProductItem:
        
        Family      Fetures
        A
                A1
                A2
                A3
        
        B
                B1
                B2
                B3
        
        C
                C1
                C2
                C3
        
        Rules:
            A3 Includes B2:  This is an inclusion rule stating that if the configuration has the feature A3 (of the
        family A) then include the feature B2 as well (of the family B).
        B2 Excludes C2:  This is an exclusion rule state that if the configuration has the feature B2 (of the family B)
        then exclude/remove the feature C2 (of the family C)
        
        Use case 1:
        Operation getAvailableProductVariability is invoked with AvailableProductVariabilityInput populated as:
        CriteriaExpression = {}
        familiesToTest = {A, B, C}
        context containing reference to Item MyProductItem
        applyConstraints = 1(enable configurator constraint evaluation)
        
        Response would contain - A1, A2, A3, B1, B2, B3, C1, C2 &amp; C3
        
        Use case 2:
        Operation getAvailableProductVariability is invoked with AvailableProductVariabilityInput populated as:
        CriteriaExpression = {A=A3}
        FamiliesToTest = {B, C}
        context containing reference to Item MyProductItem
        applyConstraints = 1
        
        Response would - B2, C1 &amp; C3
        
        Use case 3:
        Operation getAvailableProductVariability is invoked with AvailableProductVariabilityInput populated as:
        criteriaExpression = {A=A1}
        FamiliesToTest = {B, C}
        context containing reference to Item MyProductItem
        applyConstraints = 1
        
        Response would - B1, B2, B3, C1, C2 &amp; C3
        
        
        Use case 4:
        Initialize a variant criteria dialog for a new VariantRule in the Configurator context MyProductItem:
        The application uses operation getAvailableProductVariability with AvailableProductVariabilityInput populated
        as below:
        criteriaExpression = {}
        familiesToTest = {}
        context containing reference to Item MyProductItem
        ApplyConstraints = 0
        
        A service exception will be thrown by the  getAvailableProductVariability operation as familiesToTest is empty.
        """
        return cls.execute_soa_method(
            method_name='getAvailableProductVariability',
            library='Internal-Configurator',
            service_date='2015_03',
            service_name='ConfiguratorManagement',
            params={'input': input},
            response_cls=AvailableProductVariabilityOutput,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine, BOMWindow, BusinessObject
from enum import Enum
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class FormAttributeExpression(TcBaseObj):
    """
    FormAttributeExpression contains information to search
    on a single Form Attribute within a containing SearchExpressionInfo
    object.
    The search on the multiple values specified within one FormAttributeExpression are
    combined with a logical 'OR' operation.
    
    :var isItemForm: If true, Form associted with Item is to be searched; otherwise, Form associted with ItemRevision
    is searched.
    :var relationType: Relation type object with which the Form is associated with the source Item or ItemRevision.
    :var formType: Form type object to be searched.
    :var attributeName: The attribute of the above Form type to be searched.
    :var attributeType: The type of the attribute to be searched on. The supported AttributeType are: BooleanType,
    IntegerType, DoubleType, StringType, DateType and TagType.
    :var queryOperator: Operator to use for search value comparison .  The supported QueryOperator are: Equal,
    GreaterThan, GreaterThanOrEqual, LessThan, LessThanOrEqual, NotEqual, IsNull, IsNotNull, Like and NotLike.
    :var values: The values of the attribute to be searched.
    """
    isItemForm: bool = False
    relationType: BusinessObject = None
    formType: BusinessObject = None
    attributeName: str = ''
    attributeType: AttributeType = None
    queryOperator: QueryOperator = None
    values: AttributeValues = None


@dataclass
class InClassExpression(TcBaseObj):
    """
    InClassExpression
    
    :var inClassClassNames: A list of class names of the classification class used for search.
    :var inClassAttributeIDs: A list of attribute ID(s) of the class selected for classification search on which search
    needs to be performed.
    :var inClassAttributeValues: A list of values specified for search comparison.
    """
    inClassClassNames: List[str] = ()
    inClassAttributeIDs: List[int] = ()
    inClassAttributeValues: List[str] = ()


@dataclass
class AttributeExpression(TcBaseObj):
    """
    AttributeExpression encapsulates information to build a query on the attribute of an Item or ItemRevision or their
    sub-classes in the structure search scope. A set of these attribute expressions is contained inside a
    SearchExpressionInfo object. Search on multiple values specified within one AttributeExpression are combined with a
    logical 'OR' operation.
    
    :var className: Name of class, Item or ItemRevision or any of their sub-classes.
    :var attributeName: The attribute of the above 'className' to be searched on.
    :var attributeType: The type of the attribute to be searched on. The supported AttributeType are: BooleanType,
    IntegerType, DoubleType, StringType, DateType and TagType.
    :var queryOperator: Operator to use for search value comparison . The supported QueryOperator are: Equal,
    GreaterThan, GreaterThanOrEqual, LessThan, LessThanOrEqual, NotEqual, IsNull, IsNotNull, Like and NotLike.
    :var values: Values to be searched.
    """
    className: str = ''
    attributeName: str = ''
    attributeType: AttributeType = None
    queryOperator: QueryOperator = None
    values: AttributeValues = None


@dataclass
class OccurrenceNoteExpression(TcBaseObj):
    """
    OccurrenceNoteExpression contains information to search on a single occurrence note type within a containing
    SearchExpressionInfo object. The search on the multiple values specified within one OccurrenceNoteExpression are
    combined with a logical 'OR' operation.
    
    :var noteType: Occurrence note type to be search.
    :var queryOperator: Operator to use for search value comparison .  The supported QueryOperator are: Equal,
    GreaterThan, GreaterThanOrEqual, LessThan, LessThanOrEqual, NotEqual, IsNull, IsNotNull, Like and NotLike.
    :var values: A list of occurrance note values to be searched.
    """
    noteType: str = ''
    queryOperator: QueryOperator = None
    values: List[str] = ()


@dataclass
class AttributeValues(TcBaseObj):
    """
    AttributeValues data structure specifies data values to be bound to the various search expressions for
    execution of the search. It encapsulates the values to be specified for each of the data types that
    can occur in any of the attribute search expressions.
    
    :var boolValues: A list of boolean values that any of the accompanying set of search expressions may use.
    :var intValues: A list of integer values that any of the accompanying set of search expressions may use.
    :var doubleValues: A list of double values that any of the accompanying set of search expressions may use.
    :var stringValues: A list of string values that any of the accompanying set of search expressions may use.
    :var dateValues: Date values that any of the accompanying set of search expressions may use.
    :var tagValues: A list of objects that any of the accompanying set of search expressions may use.
    """
    boolValues: List[bool] = ()
    intValues: List[int] = ()
    doubleValues: List[float] = ()
    stringValues: List[str] = ()
    dateValues: datetime = None
    tagValues: List[BusinessObject] = ()


@dataclass
class PlaneZone(TcBaseObj):
    """
    Planezone defines a single plane to search against.
    
    :var xValue: X value of the direction unit vector of the plane.
    :var yValue: Y value of the direction unit vector of the plane.
    :var zValue: Z value of the direction unit vector of the plane.
    :var displacement: Plane displacement in metres along the direction vector specified above.
    """
    xValue: float = 0.0
    yValue: float = 0.0
    zValue: float = 0.0
    displacement: float = 0.0


@dataclass
class PlaneZoneExpression(TcBaseObj):
    """
    PlaneZoneExpression represents a 3D plane to search against. It may define a search for all parts that intersect
    with the plane, or a search
    for all parts on one side of the plane.
    
    :var planzeZone: It defines a plane to search against.
    :var planeZoneOperator: Operator to use for expression. The supported PlaneZoneOperator are: PlaneZone_above,
    PlaneZone_below, PlaneZone_intersects, PlaneZone_aboveOrIntersects and PlaneZone_belowOrIntersects.
    """
    planzeZone: PlaneZone = None
    planeZoneOperator: PlaneZoneOperator = None


@dataclass
class ProximityExpression(TcBaseObj):
    """
    ProximityExpression specifies one set of proximity criteria. The search results from each bomLine are combined
    together using an 'OR' operation.
    
    :var bomLines: List of source BOMLine(s) for proximity search
    :var distance: Proximity distance in metres from the outer surface of the BOMLine geometry
    :var includeChildBomLines: If true, all child BOMLines of source bom lines are considered for proximity search
    """
    bomLines: List[BOMLine] = ()
    distance: float = 0.0
    includeChildBomLines: bool = False


@dataclass
class SavedQueryExpression(TcBaseObj):
    """
    SavedQueryExpression defines a search expression encapsulated as a Teamcenter saved-query that returns an Item,
    ItemRevision or any of their sub-types. The saved query may be a pre-defined out of the box saved query or one
    created using the Teamcenter QueryBuilder application.
    
    :var savedQuery: Object of an existing saved query.
    :var entries: A list of attribute entries that needs to be searched.
    :var values: A list of values of the above entries to be searched.
    """
    savedQuery: BusinessObject = None
    entries: List[str] = ()
    values: List[str] = ()


@dataclass
class SearchBox(TcBaseObj):
    """
    SearchBox defines a single box zone to search in. It contains the definition of an axis-aligned box and an
    accompanying transform to describe it orientation.
    
    :var xmin: Point defining the minimum extent along the x-axis.
    :var ymin: Point defining the minimum extent along the y-axis.
    :var zmin: Point defining the minimum extent along the z-axis.
    :var xmax: Point defining the maximum extent along the x-axis.
    :var ymax: Point defining the maximum extent along the y-axis.
    :var zmax: Point defining the maximum extent along the z-axis.
    :var transform: Transform defining the orientation of the box ( empty vector denotes identity transform ).
    """
    xmin: float = 0.0
    ymin: float = 0.0
    zmin: float = 0.0
    xmax: float = 0.0
    ymax: float = 0.0
    zmax: float = 0.0
    transform: List[float] = ()


@dataclass
class SearchBySizeExpression(TcBaseObj):
    """
    SearchBySizeExpression defines a search for parts with bounding boxes that are larger than or smaller than
    a specified size.
    
    :var largerThan: If true, it searches for larger than the length specified; otherwise, searches for smaller.
    :var diagonalLength: The length of the largest bounding box diagonal, in metres.
    """
    largerThan: bool = False
    diagonalLength: float = 0.0


@dataclass
class SearchExpressionSet(TcBaseObj):
    """
    SearchExpression data structure specifies the full set of search expressions that are to be used to perform a
    single structure search. Each one of the expressions provided are combined together using a logical 'AND' between
    them.
    
    :var itemAndRevisionAttributeExpressions: itemAndRevisionAttributeExpressions
    :var occurrenceNoteExpressions: A collection of Occurrence Note attribute search expressions
    :var returnScopedSubTreesHit: Specifies whether the search result should return only the sub-set of scopeBomLines
    given as input as part of the SearchScope
    :var formAttributeExpressions: A collection of Form attribute search expressions
    :var proximitySearchExpressions: A collection of spatial proximity search expressions
    :var boxZoneExpressions: A collection of spatial box zone expressions
    :var planeZoneExpressions: A collection of spatial plane zone search expressions
    :var savedQueryExpressions: A collection of saved query search expressions
    :var inClassQueryExpressions: inClassQueryExpressions
    :var sizeSearchExpression: sizeSearchExpression
    :var doTrushapeRefinement: Specifies whether to perform trueshape refinement on the spatial search or not
    """
    itemAndRevisionAttributeExpressions: List[AttributeExpression] = ()
    occurrenceNoteExpressions: List[OccurrenceNoteExpression] = ()
    returnScopedSubTreesHit: bool = False
    formAttributeExpressions: List[FormAttributeExpression] = ()
    proximitySearchExpressions: List[ProximityExpression] = ()
    boxZoneExpressions: List[BoxZoneExpression] = ()
    planeZoneExpressions: List[PlaneZoneExpression] = ()
    savedQueryExpressions: List[SavedQueryExpression] = ()
    inClassQueryExpressions: List[InClassExpression] = ()
    sizeSearchExpression: SearchBySizeExpression = None
    doTrushapeRefinement: bool = False


@dataclass
class SearchScope(TcBaseObj):
    """
    The SearchScope specifies the scope of a search
    
    :var window: BOM window object specifying a structure context within which to perform search
    :var scopeBomLines: List of BOMLine objects, specifying scope of the search
    :var ignoreOccurrenceTypes: List of strings, specifying occurrence types to be ignored from search results
    """
    window: BOMWindow = None
    scopeBomLines: List[BOMLine] = ()
    ignoreOccurrenceTypes: List[str] = ()


@dataclass
class BoxZoneExpression(TcBaseObj):
    """
    BoxZoneExpression specifies one set of Box zone criteria. The search results from each SearchBox are combined
    together using an 'OR' operation.
    
    :var searchBoxes: List of 'SearchBox', specifying box(s) around which to search.
    :var boxOperator: Operator to use for expression. The supported BoxOperator are: Inside, Outside, Intersects,
    InsideOrIntersects and OutsideOrIntersects.
    """
    searchBoxes: List[SearchBox] = ()
    boxOperator: BoxOperator = None


@dataclass
class StructureSearchResultResponse(TcBaseObj):
    """
    StructureSearchResultResponse is the result set received from a single step in a structure search.
    
    :var serviceData: An object of 'ServiceData' which contains any exceptions if occurred during search.
    :var bomLines: A batch of BOMLine(s) that satisfy the search criteria for this search. Not used for 'stopSearch'.
    :var linesDone: The number of lines returned in the response. Not used for 'stopSearch'.
    :var estimatedLinesLeft: An estimate of the number of lines yet to be processed. Not used for 'startSearch',
    'nextSearch 'and' stopSearch'.
    :var percentComplete: Not used for 'startSearch', 'nextSearch' and 'stopSearch'.
    :var searchCursor: Search cursor object that holds the current state of the search in the BOM session. Not used for
    'stopSearch'.
    :var finished: A boolean value 'true' specifies that the search has stopped.
    """
    serviceData: ServiceData = None
    bomLines: List[BOMLine] = ()
    linesDone: int = 0
    estimatedLinesLeft: int = 0
    percentComplete: float = 0.0
    searchCursor: BusinessObject = None
    finished: bool = False


class AttributeType(Enum):
    """
    AttributeType enumerated type defines the type of data passed in an attribute search expression
    """
    BooleanType = 'BooleanType'
    IntegerType = 'IntegerType'
    DoubleType = 'DoubleType'
    StringType = 'StringType'
    DateType = 'DateType'
    TagType = 'TagType'


class PlaneZoneOperator(Enum):
    """
    PlaneZoneOperator enumerated type defines the operator to use as part of a plane-zone search expression
    """
    Plane_above = 'Plane_above'
    Plane_below = 'Plane_below'
    Plane_intersects = 'Plane_intersects'
    Plane_aboveOrIntersects = 'Plane_aboveOrIntersects'
    Plane_belowOrIntersects = 'Plane_belowOrIntersects'


class QueryOperator(Enum):
    """
    QueryOperator  enumerated type defines the operator to be used for comparison in a attribute search expression
    """
    Equal = 'Equal'
    GreaterThan = 'GreaterThan'
    GreaterThanOrEqual = 'GreaterThanOrEqual'
    LessThan = 'LessThan'
    LessThanOrEqual = 'LessThanOrEqual'
    NotEqual = 'NotEqual'
    IsNull = 'IsNull'
    IsNotNull = 'IsNotNull'
    Like = 'Like'
    NotLike = 'NotLike'


class BoxOperator(Enum):
    """
    BoxOperator defines the operator to apply as part of a box-zone search expression.
    """
    Inside = 'Inside'
    Outside = 'Outside'
    Intersects = 'Intersects'
    InsideOrIntersects = 'InsideOrIntersects'
    OutsideOrIntersects = 'OutsideOrIntersects'



from __future__ import annotations

from tcsoa.gen.Internal.Rdv._2009_01.VariantManagement import MetaExprTokens, GetAbeBomlineChildComponentsResponse, GetABandABEResponse, GetAbeMeapnChildComponentsResponse, NVEMetaExpressionResponse, NVEMetaToken, GetABandABEInputInfo, GetAbeChildComponentsInputInfo, ValidateNVEMetaExprResponse, AddDesignToProductResponse
from tcsoa.gen.BusinessObjects import BOMLine, BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class VariantManagementService(TcService):

    @classmethod
    def reapplyNVEMetaExpressions(cls, bomlines: List[BOMLine], validate: bool) -> ServiceData:
        """
        This operation reapplies the MetaExpression to the set of BOMLine objetcs. Meta expressions are used to balance
        the tree structure amongst the variant expressions and validate the input combination of Named Variant
        Expressions (NVEs). MetaExpression is the internal representation of Thought. If input parameter 'validate' is
        true then the MetaExpression will be validated before applying to the BOMLine. This operation gets the
        NVEMetaExpression object from the BOMLine. NVEMetaExpression object get 'validated' if validate flag is true
        after that NVEMetaExpression object reapplied to each BOMLine.
        
        Use cases:
        This operation takes the list of BOMLine objects on which metaexpression needs to be reapplied. Meta expression
        can be validated by setting the 'validate' flag to true.
        """
        return cls.execute_soa_method(
            method_name='reapplyNVEMetaExpressions',
            library='Internal-Rdv',
            service_date='2009_01',
            service_name='VariantManagement',
            params={'bomlines': bomlines, 'validate': validate},
            response_cls=ServiceData,
        )

    @classmethod
    def replaceDesignInProduct(cls, component: BusinessObject, bomlinesToReplace: List[BusinessObject], prodRevs: List[BusinessObject], archApn: BusinessObject, lous: List[BusinessObject], metaExpr: List[NVEMetaToken], validate: bool) -> ServiceData:
        """
        This operation applies the variant data to the occurrence of the already added design solution to the
        Installation Assembly. There is no need to associate the Installation Assembly with the architecture as this
        has already been done. The purpose is simply to apply the appropriate variant data to the occurrence of the
        design solution.
        If the number of 'bomlinesToReplace' is 1, then the input component replaces the Item or ItemRevision  on that
        target BOMLine, and the meta expression is applied. If multiple 'bomlinesToReplace' are provided, no substitute
        occurs, but the meta expression is still applied.
        
        Use cases:
        Use case 1: Replacing a design solution with already added design solution
        User can replace existing design solution with a new one or modify the variant condition by specifying its Item
        or ItemRevision in the 'component' parameter and also specify the new variant condition in 'metaExpr'
        parameter. In this case, size of 'bomlinesToReplace' list should be 1.
        
        Use case 2. Updating variant data of multiple design solutions
        If user needs to modify the variant conditions of multiple design solutions in one go, this can be done by
        specifying variant condition data using 'metaExpr' parameter and the multiple designs Item Revisions in
        'prodRevs' parameter. The number of design solutions whose variant data is to be updates is represented by
        'bomlinesToReplace' parameter and its size should be greater than 1. In this case the 'component' parameter is
        ignored.
        """
        return cls.execute_soa_method(
            method_name='replaceDesignInProduct',
            library='Internal-Rdv',
            service_date='2009_01',
            service_name='VariantManagement',
            params={'component': component, 'bomlinesToReplace': bomlinesToReplace, 'prodRevs': prodRevs, 'archApn': archApn, 'lous': lous, 'metaExpr': metaExpr, 'validate': validate},
            response_cls=ServiceData,
        )

    @classmethod
    def addDesignToProduct(cls, component: BusinessObject, iaRev: BusinessObject, prodRevs: List[BusinessObject], archApn: BusinessObject, lous: List[BusinessObject], noRequestedOccs: int, metaExpr: List[NVEMetaToken], validate: bool) -> AddDesignToProductResponse:
        """
        This operation adds a new design solution to a CAD structure node, referred as Installation Assembly (IA), and
        applies variant conditions on the newly added design assembly. The variant conditions are constructed from the
        Named Variant Expressions (NVE) available to the architecture breakdown element that is associated with the
        Installation Assembly. It is assumed the user has already created the design solution and needs to add it to
        the appropriate parent Installation Assembly. 
        Meta expressions are used to balance the tree structure amongst the variant expressions and validate the input
        combination of NVEs.
        All the parameter values are mandatory, except the 'lous' and 'metaExpr' which are optional.
        
        Use cases:
        Use Case 1: Add Design to Product
        This operation is called whenever a user wants to add a design solution to a CAD structure (referred as
        installation assembly). Prior to calling this operation, user is expected to associate the installation
        assembly to architecture breakdown element. After the operation execution the design assembly ('iaRev'
        parameter) is added to the product structure ('prodRevs' parameter) and the variant conditions if supplied
        (using the 'metaExpr' parameter) is applied on the occurrence of the design solution.
        """
        return cls.execute_soa_method(
            method_name='addDesignToProduct',
            library='Internal-Rdv',
            service_date='2009_01',
            service_name='VariantManagement',
            params={'component': component, 'iaRev': iaRev, 'prodRevs': prodRevs, 'archApn': archApn, 'lous': lous, 'noRequestedOccs': noRequestedOccs, 'metaExpr': metaExpr, 'validate': validate},
            response_cls=AddDesignToProductResponse,
        )

    @classmethod
    def validateNVEMetaExpressions(cls, metaExprs: List[MetaExprTokens]) -> ValidateNVEMetaExprResponse:
        """
        This operation validates the list of 'MetaExprTokens'. 'MetaExprTokens' contains the list of 'NVEMetaToken'.
        'MetaExprTokens' is the internal representation of the Thought.
        
        Meta expressions are used to balance the tree structure amongst the variant expressions and validate the input
        combination of NamedVariantExpressions. This operation creates the NVEMetaExpression object from the list of
        'NVEMetaToken' and then NVEMetaExpression object gets validated.
         
        Validation is the process to check if input meta expression always evaluates to 'true' or 'false', in this case
        validation will fail otherwise it will pass. This operation also fails if metaExprs contains invalid
        'MetaExprTokens' objects.
        
        
        Use cases:
        Validating a Thought
        
        Validation of a Thought is performed by converting the Thought into meta expression which is provided as an
        input to this operation.
        """
        return cls.execute_soa_method(
            method_name='validateNVEMetaExpressions',
            library='Internal-Rdv',
            service_date='2009_01',
            service_name='VariantManagement',
            params={'metaExprs': metaExprs},
            response_cls=ValidateNVEMetaExprResponse,
        )

    @classmethod
    def applyNVEMetaExpression(cls, bomlines: List[BOMLine], metaExpr: List[NVEMetaToken], validate: bool) -> ServiceData:
        """
        This operation applies the given MetaExpression to the set of BOMLine objects. Meta expressions are used to
        balance the tree structure amongst the variant expressions and validate the input combination of Named Variant
        Expressions (NVEs). MetaExpression is the internal representation of the Thought. If the input parameter
        validate is true then the MetaExpression will be validated before applying on the BOMLine. This operation
        creates the NVEMetaExpression object from the list of NVEMetaToken objects. The operation fails if input
        MetaExpression is invalid.
        
        Use cases:
        Applying Thought to set of BOMLine objects with or without validation
        In order to apply Thought with validation user can set input argument 'validate' to true. To apply Thought
        without validation user can set 'validate' to false.
        """
        return cls.execute_soa_method(
            method_name='applyNVEMetaExpression',
            library='Internal-Rdv',
            service_date='2009_01',
            service_name='VariantManagement',
            params={'bomlines': bomlines, 'metaExpr': metaExpr, 'validate': validate},
            response_cls=ServiceData,
        )

    @classmethod
    def askNVEMetaExpression(cls, bomlines: List[BOMLine]) -> NVEMetaExpressionResponse:
        """
        This operation creates the 'MetaExprTokens' object corresponding to the BOMLine. MetaExprTokens contains the
        list of 'NVEMetaToken' which represents a token. Meta expressions are used to balance the tree structure
        amongst the variant expressions and validate the input combination of Named Variant Expressions (NVEs). Meta
        expressions are stored in the variant expression block contained in absoccdata associated with bomline. This
        operation fails if input parameter bomlines contain invalid bomline. It returns the 'NVEMetaExpressionResponse'
        object which encapsulates the list of 'MetaExprTokens' and 'ServiceData' objects.
        
        Use cases:
        Use Case 1: Getting a Thought
        This operation can be used to get the Thought associated with the BOMLine, it returns the MetaExprTokens which
        can be converted into a Thought.
        
        Use Case 2: Validating a Thought
        This operation can also be used to validate a Thought associated with the BOMLine, it returns the
        MetaExprTokens which is the internal representation of the Thought. MetaExprTokens can be validated at the
        server side.
        """
        return cls.execute_soa_method(
            method_name='askNVEMetaExpression',
            library='Internal-Rdv',
            service_date='2009_01',
            service_name='VariantManagement',
            params={'bomlines': bomlines},
            response_cls=NVEMetaExpressionResponse,
        )

    @classmethod
    def getApnComponents(cls, inputs: GetABandABEInputInfo) -> GetABandABEResponse:
        """
        This operation verifies if an Architecture Breakdown Element (ABE) exists in the given Architecture assembly or
        not for the given input generic component ID (could be a wildcard string input). If ABE exists then the
        operation returns the MEAPN(s) of the component(s) in a 'GetABandABEResponse' response object, else the
        response object is empty. Any failure in the operation is returned as a full/partial error added to the
        'ServiceData' object contained in the response object.
        
        Use cases:
        Use Case 1: Search Architecture Breakdown Element with given criteria
        This operation helps user to find ABE(s) by providing the search criteria in 'GetABandABEInputInfo' object,
        only single 'GetABandABEInputInfo' object as an input is supported by this operation. The operation returns one
        or more matching APNs in the resultant 'GetABandABEResponse' object containing the details of each APN.
        """
        return cls.execute_soa_method(
            method_name='getApnComponents',
            library='Internal-Rdv',
            service_date='2009_01',
            service_name='VariantManagement',
            params={'inputs': inputs},
            response_cls=GetABandABEResponse,
        )

    @classmethod
    def getArchbreakdownBomlineChildComponents(cls, inputs: GetAbeChildComponentsInputInfo) -> GetAbeBomlineChildComponentsResponse:
        """
        This operation retrieves the child MEAppearancePathNodes (MEAPN) and their respective BOMLine objects for a
        given parent MEAppearancePathNode. The 'topLevelItem' passed as part of the input structure is the reference to
        Architecture Breakdown which is the topline in Architecture Breakdown structure.
        
        The MEAPN passed as part of the input structure is the MEAppearancePathNode of the parent whose child
        MEAppearancePathNodes are to be retrieved. This operation fails if 'topLevelItem' is not supplied or it is not
        of type ArchitectureImpl. This operation fails if 'bomWindow' in the input structure is not supplied.
        
        
        Use cases:
        This operation can be used to get the child MEAppearancePathNodes and their respective BOMLine objects by
        providing reference to Architecture Breakdown, MEAppearancePathNode of parent and BOMWindow object in the
        'GetAbeChildComponentsInputInfo' structure.
        """
        return cls.execute_soa_method(
            method_name='getArchbreakdownBomlineChildComponents',
            library='Internal-Rdv',
            service_date='2009_01',
            service_name='VariantManagement',
            params={'inputs': inputs},
            response_cls=GetAbeBomlineChildComponentsResponse,
        )

    @classmethod
    def getArchbreakdownMeapnChildComponents(cls, inputs: GetAbeChildComponentsInputInfo) -> GetAbeMeapnChildComponentsResponse:
        """
        This operation retrieves the child MEAppearancePathNodes (MEAPNs), their respective generic component IDs and
        description for a given parent MEAPN. If child components exist, the operation returns the child MEAPNs of the
        parent component in a 'GetAbeMeapnChildComponentsResponse' response object, else the response object is empty.
        Any failure in the operation is returned as a full/partial 'ServiceData' object contained in the response
        object.
        
        Use cases:
        Use Case 1: Get child Architecture Breakdown Element(s) for given top level architecture
        This operation helps user to get all the child ABEs of the given top level Architecture Breakdown structure
        (input provided using 'GetAbeChildComponentsInputInfo' object), only single 'GetAbeChildComponentsInputInfo'
        object as an input is supported by this operation. The operation returns all the child ABEs in the resultant
        'GetAbeMeapnChildComponentsResponse' object containing the details of each ABE.
        """
        return cls.execute_soa_method(
            method_name='getArchbreakdownMeapnChildComponents',
            library='Internal-Rdv',
            service_date='2009_01',
            service_name='VariantManagement',
            params={'inputs': inputs},
            response_cls=GetAbeMeapnChildComponentsResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessModeler._2008_06.ConditionEngine import EvaluateConditionsResponse, ConditionInput
from tcsoa.gen.BusinessModeler._2008_06.DeepCopyRules import DeepCopyInfoResponse, DeepCopyInfoKey
from typing import List
from tcsoa.base import TcService


class ConditionEngineService(TcService):

    @classmethod
    def evaluateConditions(cls, inputs: List[ConditionInput]) -> EvaluateConditionsResponse:
        """
        This operation tells the CLIPS rules engine to evaluate the expression defined on the specified Condition using
        the specified input parameter(s) defined on the ConditionParameter.  This operation takes as input a set of
        conditions along with parameters for each condition and returns a set of outputs containing the result
        (true/false) and exit code of each condition evaluation. There is a one to one correspondence between the
        elements in the input set and the elements in the output set.  This allows for evaluation of multiple
        conditions in one operation call.
        """
        return cls.execute_soa_method(
            method_name='evaluateConditions',
            library='BusinessModeler',
            service_date='2008_06',
            service_name='ConditionEngine',
            params={'inputs': inputs},
            response_cls=EvaluateConditionsResponse,
        )


class DeepCopyRulesService(TcService):

    @classmethod
    def getDeepCopyInfo(cls, keys: List[DeepCopyInfoKey]) -> DeepCopyInfoResponse:
        """
        Deep copy rules define whether objects belonging to a business object instance can be copied when a user
        performs a save as or revise operation on that instance. Deep copy rules can be applied to any business object
        type, and are inherited by children business object types. This operation gets the applicable deep copy rules
        for the given list of objects and the operation specified for each object.
        """
        return cls.execute_soa_method(
            method_name='getDeepCopyInfo',
            library='BusinessModeler',
            service_date='2008_06',
            service_name='DeepCopyRules',
            params={'keys': keys},
            response_cls=DeepCopyInfoResponse,
        )



from __future__ import annotations

from tcsoa.gen.AuthorizedDataAccess._2012_09.LicenseManagement import GetLicenseDetailsResponse, LicenseDetails
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class LicenseManagementService(TcService):

    @classmethod
    def setLicenseDetails(cls, licenseInfo: List[LicenseDetails]) -> ServiceData:
        """
        This operation creates or modifies an ADA_License business object for each LicenseDetails supplied. The
        LicenseDetails contains information for properties such as license type, license ID, category, expiry date,
        lock date, reason, in accordance with, and associated users and groups, for a given license. If a specified
        license ID already exists, the rest of the property values are updated on that license. However, if the license
        ID does not exist, then a new license of the specified type and ID will be created, and the rest of the
        properties are set on the created license. The user performing the operation will need the privilege specified
        in the ADA_license_administration_privilege site preference to create/modify an ADA License. If a user does not
        have the necessary privilege or if there is a validation error, the operation would fail and the error is
        returned in the ServiceData.
        """
        return cls.execute_soa_method(
            method_name='setLicenseDetails',
            library='AuthorizedDataAccess',
            service_date='2012_09',
            service_name='LicenseManagement',
            params={'licenseInfo': licenseInfo},
            response_cls=ServiceData,
        )

    @classmethod
    def getLicenseDetails3(cls, licenseIds: List[str]) -> GetLicenseDetailsResponse:
        """
        This operation gets the properties of anfor each ADA_License business object for each given specified in
        LicenseIds licenseIds parameter. The properties of the ADA_License business objects are supplied viareturned in
        LicenseDetails structures as part of LicenseDetailsResponse. The LicenseDetails contains information for
        properties such as license type, license ID, category, expiry date, lock date, reason, in accordance with, and
        associated users and groups, for a given license. If a the user does not have the sufficientREAD privilege to
        the any license,  if the a license ID does not exist or if there is any unexpected error of while getting
        property information, the errors are returned in the  ServiceData of LicenseDetailsResponse.
        """
        return cls.execute_soa_method(
            method_name='getLicenseDetails3',
            library='AuthorizedDataAccess',
            service_date='2012_09',
            service_name='LicenseManagement',
            params={'licenseIds': licenseIds},
            response_cls=GetLicenseDetailsResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Cfg0ConfiguratorPerspective
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ConfigurationRosterCoverage(TcBaseObj):
    """
    Response structure for getConfigurationRosterCoverage service.
    
    :var jsonResponse: String Representing JSON output.
    :var serviceData: Information about errors if any.
    """
    jsonResponse: str = ''
    serviceData: ServiceData = None


@dataclass
class ConfigurationRosterInput(TcBaseObj):
    """
    Input Data structure for getConfigurationRosterCoverage service.
    
    :var perspective: Configurator perspective details.
    Property cfg0ProductItems of perspective must contain reference to a single Configurator context item and property
    cfg0RevisonRule must be set to a RevisionRule
    :var jsonRequest: String Representing JSON input, Please read JSON schema to understand it in detail.
    """
    perspective: Cfg0ConfiguratorPerspective = None
    jsonRequest: str = ''



from __future__ import annotations

from tcsoa.gen.WProxy._2014_10.ProxyLink import CreateProxyLinkInputInfo
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class ProxyLinkService(TcService):

    @classmethod
    def createProxyLink(cls, input: List[CreateProxyLinkInputInfo]) -> ServiceData:
        """
        This operation creates ProxyLink object(s) based on the input information provided and relates it to the
        primary objects at the proxy link site.
        
        Use cases:
        When the user wants to provide information about an object to a site other than the owning site of the object,
        the user can create a proxy link for that object.  From the different site, User can access the master object
        using the proxy link.
        """
        return cls.execute_soa_method(
            method_name='createProxyLink',
            library='WProxy',
            service_date='2014_10',
            service_name='ProxyLink',
            params={'input': input},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Fnd0LicenseServer
from typing import List
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class LicenseServerInput2(TcBaseObj):
    """
    The 'LicenseServerInput2' structure contains information for properties such as license server name, host, port,
    protocol, failover servers and multiple license servers, for a given license server. A license server location is
    defined by its host and port.
    
    :var licenseServerName: The input name of the license server.
    :var hostName: The host name of the license server.
    :var licenseServerType: The type of the license server.  Valid values are: "RLS" (Regular License Server) and "ILS"
    (Integration License Server).
    :var portNumber: The port number of the license server.  This is used in combination with the hostName to connect
    to the license server.
    :var protocol: The protocol used when connecting to the license server.  The protocol can only be "TCP/IP".
    :var licenseServer: The license server object to be updated.
    :var failoverLicenseServers: A list of license server objects used as failover license servers.
    :var multipleLicenseServers: A list of license server objects used as multiple license servers.
    """
    licenseServerName: str = ''
    hostName: str = ''
    licenseServerType: str = ''
    portNumber: int = 0
    protocol: int = 0
    licenseServer: Fnd0LicenseServer = None
    failoverLicenseServers: List[Fnd0LicenseServer] = ()
    multipleLicenseServers: List[Fnd0LicenseServer] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine, StructureMapRevision
from tcsoa.gen.Internal.Cae._2011_06.StructureManagement import ExecuteRuleResponse
from tcsoa.base import TcService


class StructureManagementService(TcService):

    @classmethod
    def executeDatamap(cls, rootBOMLine: BOMLine) -> ExecuteRuleResponse:
        """
        This internal operation creates an output BOM structure given the root BOMLine of an input BOM structure loaded
        in a BOMWindow. The output BOM structure is determined by the XSLT-based Data Map rules executed against the
        input BOM structure. Data Map syntax is in compliance with the schema defined in tcsim_xslWithNode.xsd, located
        in TC_DATA.
        
        Data Map rules define the mapping between an input item type and its resulting output item type. Data Map rules
        are defined for an entire site and are stored in the datamapping.xml file located in TC_DATA. The naming of the
        data mapping file is defined by the site preference CAE_dataMapping_file.
        
        The datamapping.xml file can be configured for various domains defined as LOV objects under StructureMap
        Domains in BMIDE. To configure the domains, in the Extensions view in BMIDE, open LOV->StructureMap Domains and
        add additional domain values. This operation assumes the value of the domain as CAE.
        
        To use this operation, a well-defined datamapping.xml is required in TC_DATA.
        
        
        Use cases:
        Use Case 1: Create an output structure given a top BOMLine of the input structure
        
        Given an input BOM structure, the user can apply the Data Map rules to the BOM structure and generate a
        corresponding output BOM structure. The output BOM structure would consist of BOM line occurrences of
        ItemRevision objects as defined in the datamapping.xml file. The user can review the actions executed with the
        process log returned with the BOMViewRevision.
        """
        return cls.execute_soa_method(
            method_name='executeDatamap',
            library='Internal-Cae',
            service_date='2011_06',
            service_name='StructureManagement',
            params={'rootBOMLine': rootBOMLine},
            response_cls=ExecuteRuleResponse,
        )

    @classmethod
    def executeStructureMap(cls, rootBOMLine: BOMLine, structureMapIR: StructureMapRevision) -> ExecuteRuleResponse:
        """
        This internal operation creates an output BOM structure given the root BOMLine of an input BOM structure loaded
        in a BOMWindow. The output BOM structure is determined by a combination of XSLT-based Data Map and StructureMap
        rules executed against the input BOM structure. Data Map/StructureMap syntax is in compliance with the schema
        defined in tcsim_xslWithNode.xsd, located in TC_DATA.
        
        Data Map rules define the mapping between an input item type and its resulting output item type. Data Map rules
        are defined for an entire site and are stored in the datamapping.xml file located in TC_DATA. The name of the
        data mapping file is defined by the site preference CAE_dataMapping_file.
        
        StructureMap rules tailor the output BOM Structure. There are several rule types:
        - Filter - Removes input BOM lines (and their children) from Data Map evaluation.
        - Include - Inserts item revisions in either the input or output BOM structure as required.
        - Reuse - Retrieve existing item revision to be used in the output structure.
        - Create Collector - Reorganization rule that creates "container" item revisions to move BOMLine objects and
        sub-assemblies around.
        - Move to Collector - Reorganizational rule that moves BOMLine objects and sub-assemblies to collector
        components.
        - Collapse Single Component Assembly - Identifying sub-assemblies with single child component, elevating the
        child component to the parent sub-assembly and removing the parent sub-assembly.
        - Remove Empty Assembly - Identifying sub-assemblies with no child components and removing the empty
        sub-assembly.
        - Skip - Skips the BOMLine but still process its children.
        
        
        
        StructureMap rules are stored as an XML named reference in CAEStructureMap dataset attached to a
        StructureMapRevision. StructureMap rules are created with Simulation Process Management CAE Structure Designer.
        
        To use this operation, a well-defined datamapping.xml is required in TC_DATA and a StructureMapRevision with an
        attached CAEStructureMap dataset must exist.
        
        Use cases:
        Use Case 1: Create an output structure given a top BOMLine of the input structure along with its configuration
        Given an input BOM structure, the user can apply a StructureMap rule to the BOM structure and generate a
        corresponding output BOM structure. The output BOM structure would consist of BOMLine occurrences of
        ItemRevision objects as defined in the datamapping.xml file and would be organized by the StructureMap rules
        defined in the CAEStructureMap dataset attached to the StructureMapRevision. The user can review the actions
        executed with the process log returned with the BOMView Revision.
        """
        return cls.execute_soa_method(
            method_name='executeStructureMap',
            library='Internal-Cae',
            service_date='2011_06',
            service_name='StructureManagement',
            params={'rootBOMLine': rootBOMLine, 'structureMapIR': structureMapIR},
            response_cls=ExecuteRuleResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Dataset
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class PackageBriefcaseContentsInfo(TcBaseObj):
    """
    The input for packageBriefcaseContents operation.
    
    :var sourceSiteId: The Id of the source site.
    :var targetSiteIds: The Ids of the target sites.
    :var requestingUserName: The user name who is creating the briefcase.
    :var tcplmxmlFmsTickets: FMS tickets of the source TC XML files.
    :var exportLogFmsTickets: FMS tickets of the export log files.
    :var datasetName: This string is in the format of datasetName~~transactionId. The first part will be used as the
    name of the dataset. The second part is the transaction ID which will be used to confirm the TC XML export
    opertion. This transaction ID should be the same with the one used in ImportExport::exportObjects operation.
    """
    sourceSiteId: int = 0
    targetSiteIds: List[int] = ()
    requestingUserName: str = ''
    tcplmxmlFmsTickets: List[str] = ()
    exportLogFmsTickets: List[str] = ()
    datasetName: str = ''


@dataclass
class PackageBriefcaseContentsResponse(TcBaseObj):
    """
    The response from packageBriefcaseContents operation.
    
    :var briefcaseDataSet: The briefcase Dataset with briefcase file as named reference.
    :var briefcaseDataSetUrl: The URL of the briefcase Dataset. This is constructed from the Dataset.
    :var serviceData: Contains the partial error, the briefcase datasets are added to the createdObject list.
    """
    briefcaseDataSet: Dataset = None
    briefcaseDataSetUrl: str = ''
    serviceData: ServiceData = None


@dataclass
class UnpackBriefcaseContentsResponse(TcBaseObj):
    """
    The response from unpackBriefcaseContents operation.
    
    :var sourceSiteId: The Id of the source site.
    :var tcplmxmlFmsTickets: FMS tickets of the output TCXML files which are ready for import.
    :var serviceData: The Service Data. The input briefcase TC file will be deleted and added to deletedObject list.
    """
    sourceSiteId: int = 0
    tcplmxmlFmsTickets: List[str] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine, AbsOccurrence, BusinessObject, AbsOccData, AssemblyArrangement, PSOccurrenceThread, VariantRule, PSBOMViewRevision, Dataset, MEAppearancePathNode, BOMWindow, Form, ItemRevision
from tcsoa.gen.Cad._2007_01.StructureManagement import AttributesInfo, RevisionRuleConfigInfo, OccNote, RelationAndTypesFilter, AssemblyArrangementInfo, RelOccInfo
from enum import Enum
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class ExpandPSAllLevelsInfo(TcBaseObj):
    """
    Contains parentBomLines & excludeFilter.
    
    :var parentBomLines: List of parent bom lines that needs to be expanded
    :var excludeFilter: Filter to exclude the type of BOMLines.
    Valid values are:
    None2 -- Returns all information about the structure.
    ExcludeOverridden2 -- Excludes structure or property values that are removed by AbsOccs substitution.
    ExcludeICHistory2 -- Excludes structure (or property values) that are configured out by ICs.
    ExcludeGDEs2 -- Excludes lines that are GDEOccurrences.
    ExcludeImanItemLines2 -- Excludes any lines that are ImanItemLines.
    """
    parentBomLines: List[BOMLine] = ()
    excludeFilter: BOMLineFilter2 = None


@dataclass
class ExpandPSAllLevelsOutput(TcBaseObj):
    """
    Structure containing ExpandPSParentData2 corresponding to the parent and a list of ExpandPSData of the children
    
    :var parent: ExpandPSParentData member
    :var children: List of ExpandPSData children found for this parent.
    """
    parent: ExpandPSParentData = None
    children: List[ExpandPSData] = ()


@dataclass
class ExpandPSAllLevelsPref(TcBaseObj):
    """
    More than one filtering criteria can be specified using this which is nothing but a list of  RelationAndTypesFilter
    
    :var expItemRev: Flag to check for expanding the item revision further
    :var info: List of the relation name and the types of objects of the given relation to return along with the
    children
    :var includeOccurrenceTypes: List of Occurrence Types that needs to be included when the expansion of the BOM takes
    place.
    :var excludeOccurrenceTypes: List of Occurrence Types that needs to be excluded when the expansion of the BOM takes
    place.
    """
    expItemRev: bool = False
    info: List[RelationAndTypesFilter] = ()
    includeOccurrenceTypes: List[str] = ()
    excludeOccurrenceTypes: List[str] = ()


@dataclass
class ExpandPSAllLevelsResponse2(TcBaseObj):
    """
    A list of ExpandPSAllLevelsOutput2 so that a set of parent BOMLines can be expanded.
    
    :var output: List of ExpandPSAllLevelsOutput1 which contains ExpandPSParentData and list of ExpandPSData
    :var serviceData: The SOA framework object containing objects that were created, deleted, or updated by the
    Service, plain objects, and error information.
    For this service, all objects are returned to the plain objects group.
    Error information will also be returned mapped to input object.
    """
    output: List[ExpandPSAllLevelsOutput] = ()
    serviceData: ServiceData = None


@dataclass
class ExpandPSData(TcBaseObj):
    """
    Through this structure, the child bom line , the object of the bom line and the object attached to the bom line
    object are returned.
    
    :var bomLine: BOMline object reference of the children
    :var objectOfBOMLine: Object that the child represents
    :var relatedObjects: List of objects attached to children with given relation
    """
    bomLine: BOMLine = None
    objectOfBOMLine: BusinessObject = None
    relatedObjects: List[ExpandPSRelatedObjectInfo] = ()


@dataclass
class ExpandPSNamedReferenceInfo(TcBaseObj):
    """
    This structure is used to identify the reference object corresponding to the named reference.
    
    :var namedReferenceType: type of reference object.
    :var namedReferenceName: name of reference object.
    :var referenceObject: Object reference corresponding to the named reference.
    :var fileTicket: FMS ticket used to retrieve the file in cases where referenceObject is a file.
    """
    namedReferenceType: str = ''
    namedReferenceName: str = ''
    referenceObject: BusinessObject = None
    fileTicket: str = ''


@dataclass
class ExpandPSOneLevelInfo(TcBaseObj):
    """
    Contains the parent BOM lines whose children are to be expanded.
    
    :var parentBomLines: List of parent BOM lines to be expanded.
    :var excludeFilter: A filter used to identify the type of BOM lines to exclude.
    Valid values are: None2 -- Returns all information about the structure.
    ExcludeOverridden2 -- Excludes structure or property values that are removed by AbsOccs subsititution.
    ExcludeICHistory2 -- Excludes structure that are configured out by ICs.
    ExcludeGDEs2 -- Excludes lines that are GDEOccurrences.
    ExcludeImanItemLines2 -- Excludes lines that are ImanItemLines.
    """
    parentBomLines: List[BOMLine] = ()
    excludeFilter: BOMLineFilter2 = None


@dataclass
class ExpandPSOneLevelOutput(TcBaseObj):
    """
    Used to return a parent bomline and its related data, and a list of bomlines and related data that share that
    parent.
    
    :var parent: parent data member
    :var children: list of children returned for the parent
    """
    parent: ExpandPSParentData = None
    children: List[ExpandPSData] = ()


@dataclass
class ExpandPSOneLevelPref(TcBaseObj):
    """
    Contains a list of filtering criteria (RelationAndTypesFilter) used in a product structure expand operation.
    
    :var expItemRev: Flag indicating whether to return related object(s).
    :var info: List of the relation name and the related object types to return along with the children.  If no
    RelationAndTypesFilter is supplied (info is empty), and expItemRev is true, then all related object types are to be
    accepted.
    :var includeOccurrenceTypes: List of Occurrence Types that needs to be included when the expansion of the BOM takes
    place.
    :var excludeOccurrenceTypes: List of Occurrence Types that needs to be excluded when the expansion of the BOM takes
    place.
    """
    expItemRev: bool = False
    info: List[RelationAndTypesFilter] = ()
    includeOccurrenceTypes: List[str] = ()
    excludeOccurrenceTypes: List[str] = ()


@dataclass
class ExpandPSOneLevelResponse2(TcBaseObj):
    """
    Is the response object returned in a product structure one level expand operation.
    
    :var output: List of ExpandPSOneLevelOutput structures.
    :var serviceData: The SOA framework object containing objects that were created, deleted, or updated by the
    Service, plain objects, and error information.
    For this service, all objects are returned to the plain objects group.
    Error information will also be returned.
    """
    output: List[ExpandPSOneLevelOutput] = ()
    serviceData: ServiceData = None


@dataclass
class ExpandPSParentData(TcBaseObj):
    """
    Through this structure, the parent bom line , the object of the bom line and the objects attached to the bom line
    object are returned.
    
    :var bomLine: BOMline object reference of the parent
    :var objectOfBOMLine: Object that the parent represents
    :var parentRelatedObjects: List of object references attached to parent with given relation
    """
    bomLine: BOMLine = None
    objectOfBOMLine: BusinessObject = None
    parentRelatedObjects: List[ExpandPSRelatedObjectInfo] = ()


@dataclass
class ExpandPSRelatedObjectInfo(TcBaseObj):
    """
    This structure associates a related object, named references and reference objects.
    
    :var relatedObject: The resulting related object by following a relation specified in the ExpandPSOneLevelPref
    :var namedRefList: List of named reference and reference object of relatedObject.
    """
    relatedObject: BusinessObject = None
    namedRefList: List[ExpandPSNamedReferenceInfo] = ()


@dataclass
class ApnToThreadPathStruct(TcBaseObj):
    """
    Contains the appearance path node and the occurrence thread path corresponding to the apn.
    
    :var apn: The appearance path node
    :var occThreadPath: The occurrence thread path corresponding to the apn
    """
    apn: MEAppearancePathNode = None
    occThreadPath: List[PSOccurrenceThread] = ()


@dataclass
class FormInfo(TcBaseObj):
    """
    Form information for the occurrence.
    
    :var formObject: The form object reference.  This input can be null for create, but it is required for update.
    :var name: The form name attribute value.
    :var description: The form description attribute value.
    :var formType: The form type name.
    :var attributes: The attributes to be set or updated on the form.
    """
    formObject: Form = None
    name: str = ''
    description: str = ''
    formType: str = ''
    attributes: List[NameValueStruct] = ()


@dataclass
class GetAbsoluteStructureDataResponse(TcBaseObj):
    """
    Contains list of overrides and serviceData.
    
    :var overrides: List of overrides
    :var serviceData: Contains populated plain object list and partial errors.
    """
    overrides: List[AbsOccStructureDataInfo] = ()
    serviceData: ServiceData = None


@dataclass
class AbsOccAttachment2(TcBaseObj):
    """
    Contains an override object which is attached as the override data and a 'relationTypeName' to relate the
    'AbsOccData' to the object.
    
    :var overrideObject: The object which is attached as the override data.
    :var relationTypeName: Relation name or property name to relate 'AbsOccData' to object.
    """
    overrideObject: BusinessObject = None
    relationTypeName: str = ''


@dataclass
class AbsOccCreateDatasetAttachmentInfo(TcBaseObj):
    """
    Dataset information for attaching a dataset to the absolute occurrence.
    
    :var clientId: A unique string supplied by the caller.  This ID is used to identify return data elements and
    partial errors associated with this input structure.  If 'clientId' is not provided then it can be difficult to
    align the input with any returned errors.
    :var datasetInfo: The information used to create or update the dataset.
    :var relationTypeName: The relation type used for the dataset attached to the absolute occurrence.
    :var createIfFound: Flag to specify whether to create, update or attach the specified dataset if the supplied type
    and relation already exist as an override.
    """
    clientId: str = ''
    datasetInfo: DatasetInfo = None
    relationTypeName: str = ''
    createIfFound: bool = False


@dataclass
class AbsOccCreateFormAttachmentInfo(TcBaseObj):
    """
    Form information for attaching a form to the absolute occurrence.
    
    :var clientId: A unique string supplied by the caller.  This ID is used to identify return data elements and
    partial errors associated with this input structure.  If 'clientId' is not provided then it can be difficult to
    align the input with any returned errors.
    :var formInfo: The information used to create or update the form.
    :var relationTypeName: The relation type used for the form attached to the absolute occurrence.
    :var createIfFound: Flag to specify whether to create, update or attach the specified form if the supplied type and
    relation already exist as an override.
    """
    clientId: str = ''
    formInfo: FormInfo = None
    relationTypeName: str = ''
    createIfFound: bool = False


@dataclass
class AbsOccDataGRMExpansionInfo(TcBaseObj):
    """
    Contains a relationName and secondary objects attached as a override with the given relation.
    
    :var relationName: The relation name found for the override.
    :var objects: The secondary objects attached as a override with the given relation.
    """
    relationName: str = ''
    objects: List[BusinessObject] = ()


@dataclass
class AbsOccDataInfo2(TcBaseObj):
    """
    Detailed attribute information for the occurrence.
    
    :var overridesToSet: List of attribute information to set as overrides for the occurrence.
    :var overridesToRemove: List of attribute names that will be unset or removed as overrides for the occurrence.  For
    example, to remove a transform override, add the attribute name for the transform to this list.
    :var usedArr: Reference of existing used arrangement for this absolute occurrence.
    :var boundingBoxInfo: No longer used. Bounding Box information is now passed in through 'datasetAttachments'.
    :var asRequired: Used to set the quantity as required occurrence flag.
    :var occTransform: Positioning information for the occurrence.
    :var occNotes: List of note information for the occurrence.
    :var attachments: Attachments to add to the occurrence.
    :var attachmentsToUnattach: Attachments to remove from the occurrence.
    :var datasetAttachments: Dataset attachments to create or update for the occurrence.
    :var formAttachments: Form attachments to create or update for the occurrence.
    :var clientIdOfUsedArrangement: Client ID of the used arrangement created for this absolute occurrence.
    """
    overridesToSet: List[AttributesInfo] = ()
    overridesToRemove: List[str] = ()
    usedArr: AssemblyArrangement = None
    boundingBoxInfo: List[BoundingBoxInfo] = ()
    asRequired: bool = False
    occTransform: List[float] = ()
    occNotes: List[OccNote] = ()
    attachments: List[AbsOccAttachment2] = ()
    attachmentsToUnattach: List[AbsOccAttachment2] = ()
    datasetAttachments: List[AbsOccCreateDatasetAttachmentInfo] = ()
    formAttachments: List[AbsOccCreateFormAttachmentInfo] = ()
    clientIdOfUsedArrangement: str = ''


@dataclass
class AbsOccDataPref(TcBaseObj):
    """
    Contains a filter that needs to be applied when expanding a GRM override and the filter that needs to be applied on
    the data that is returned based on the qualifier.
    
    :var relationFilterInfos: The filter that needs to be applied when expanding a GRM override.
    :var qualifierFilter: The filter that needs to be applied on the data that is returned based on the qualifier.
    Legal values are : 'None' 'IncludeBvrOnlyQualifyingOverrides' 'IncludeBvrAndUpperQualifyingOverrides' 
    """
    relationFilterInfos: List[RelationAndTypesFilter] = ()
    qualifierFilter: AbsOccDataQualifierFilter = None


@dataclass
class NameValueStruct(TcBaseObj):
    """
    Contains name and list of value strings.
    
    :var name: Attribute name to set
    :var values: Attribute value to set
    """
    name: str = ''
    values: List[str] = ()


@dataclass
class NamedReferenceObjectInfo(TcBaseObj):
    """
    Contains clientId, Object reference, namedReferenceName, typeName and dataNameValuePairs.
    
    :var clientId: Unique identifier to track the related object.
    :var object: Object reference of the object for update, null for create
    :var namedReferenceName: The Named Reference from the dataset to this object, required. NamedReference values  are
    defined for each Dataset type. The customer can add more values as needed. To get a current list of valid Named
    Reference values the programmer can either use BMIDE or can call the SOA Core service getDatasetTypeIno.
    :var typeName: Type of the object to be created. Required for object creation only.
    :var dataNameValuePairs: List of NameValueStruct.
    """
    clientId: str = ''
    object: BusinessObject = None
    namedReferenceName: str = ''
    typeName: str = ''
    dataNameValuePairs: List[NameValueStruct] = ()


@dataclass
class OptionsInfo(TcBaseObj):
    """
    This contains classic variant option information.
    
    :var optionName: Refers to classic variant option name.
    :var optionValue: Refers to classic variant option value.
    :var assocRev: Refers to the ItemRevision object on which the classic variant options are defined.
    """
    optionName: str = ''
    optionValue: str = ''
    assocRev: ItemRevision = None


@dataclass
class AbsOccInfo2(TcBaseObj):
    """
    Attribute information for the occurrence.
    
    :var clientId: A unique string supplied by the caller.  This ID is used to identify return data elements and
    partial errors associated with this input structure.  If 'clientId' is not provided then it can be difficult to
    align the input with any returned errors.
    :var absOcc: Absolute occurrence object reference.  This input can be null for create.
    :var cadOccIdPath: Path of CAD occurrence IDs that identify or find the BOM line.
    :var absOccData: Override data to set for the absolute occurrence.
    """
    clientId: str = ''
    absOcc: AbsOccurrence = None
    cadOccIdPath: List[str] = ()
    absOccData: AbsOccDataInfo2 = None


@dataclass
class AbsOccQualifierInfo(TcBaseObj):
    """
    Contains the context BVR for which the overrides are to be retrieved and an upperQualifier.
    
    :var qualifyingBVR: The context BVR for which the overrides are to be retrieved.
    :var upperQualifier: Context object of the override (such as Arrangement).
    """
    qualifyingBVR: PSBOMViewRevision = None
    upperQualifier: BusinessObject = None


@dataclass
class AbsOccStructureDataInfo(TcBaseObj):
    """
    Contains the list of apn and occurrence thread path that the override are applied. The occurrence thread paths that
    are returned & dataOverrideInfo.
    
    :var occThreadPaths: The list of apn and occurrence thread path that the override are applied.
    The occurrence thread paths that are returned for the given override are in top down order.
    Since the operation returns unconfigured data, the client needs to match the thread paths in previously expanded
    context with the returned ones to determine the exact override.
    :var dataOverrideInfo: dataOverrideInfo
    """
    occThreadPaths: List[ApnToThreadPathStruct] = ()
    dataOverrideInfo: DataOverrideInfo = None


@dataclass
class ReconfigureBOMWindowsInfo(TcBaseObj):
    """
    This contains the list of BOMWindow objects and corresponding corresponding RevisionRule object and VariantRule
    object or StoredOptionSet object information.
    
    :var clientID: Used to track the object(s) created
    :var bomWindow: The BOMWindow object which needs to be reconfigured.
    :var objectForConfigure: Refers to an VariantRule object or StoredOptionSet object information.
    :var revRuleConfigInfo: Refers to an 'RevisionRuleConfigInfo' struct object.
    """
    clientID: str = ''
    bomWindow: BOMWindow = None
    objectForConfigure: BusinessObject = None
    revRuleConfigInfo: RevisionRuleConfigInfo = None


@dataclass
class ReconfigureBOMWindowsOutput(TcBaseObj):
    """
    Contains updated BOMWindow along with the corresponding clientIds.
    
    :var clientID: clientID
    :var bomWindow: The reconfigured BOMWindow
    """
    clientID: str = ''
    bomWindow: BOMWindow = None


@dataclass
class ReconfigureBOMWindowsResponse(TcBaseObj):
    """
    This contains the response for the 'reconfigureBOMWindows' operation.
    
    :var output: This contains list of 'ReconfigureBOMWindowsOutput' struct object, which returns the BOMWindow which
    has updated configuration.
    :var serviceData: The SOA framework object containing objects that were created, deleted, or updated by the service
    and error information if any.
    """
    output: List[ReconfigureBOMWindowsOutput] = ()
    serviceData: ServiceData = None


@dataclass
class RelatedObjectTypeAndNamedRefs(TcBaseObj):
    """
    This structure contains a related object type and the list of its named references to be processed.
    
    :var objectTypeName: objectTypeName
    :var namedReferenceNames: namedReferenceNames
    """
    objectTypeName: str = ''
    namedReferenceNames: List[str] = ()


@dataclass
class RelationAndTypesFilter(TcBaseObj):
    """
    This structure contains a relation name and a list of related object types and its named references
    (RelatedObjectTypeAndNamedReferences).  Together they are used to filter objects passed back in a product structure
    expand operation.
    
    :var relationName: Relation name.
    :var relatedObjAndNamedRefs: List of related object types and named references.
    :var namedRefHandler: An enumeration used to identify how named references will be processed.
    Valid values are:
    NoNamedRefs -- No named references are to be processed. The input RelatedObjectTypeAndNamedRefs will be ignored.
    AllNamedRefs -- All named references are to be processed. The input RelatedObjectTypeAndNamedRefs will be ignored.
    UseNamedRefsList -- Only the named references listed in the input RelatedObjectTypeAndNamedRefs struct are
    processed.
    PreferredJT -- Specialized code for selecting which named references to process is executed.
    This is intended for selecting the most appropriate JT files for visualization purposes.
    If related object is a DirectModel, RelatedObjectTypeAndNamedReferences contents will be ignored and only the
    preferred JT is returned.
    If related object is not Direct Model, named reference expansion will proceed as though namedRefHandler is
    UseNamedRefsList.
    """
    relationName: str = ''
    relatedObjAndNamedRefs: List[RelatedObjectTypeAndNamedRefs] = ()
    namedRefHandler: NamedRefHandler = None


@dataclass
class RelativeStructureChildInfo2(TcBaseObj):
    """
    Contains information about the child structure.
    
    :var clientId: A unique string supplied by the caller.  This ID is used to identify return data elements and
    partial errors associated with this input structure.  If 'clientId' is not provided then it can be difficult to
    align the input with the output or any returned errors.
    :var cadOccId: The CAD occurrence ID or PSOccurrenceThread UID is used to uniquely identify the occurrence under a
    particular context item revision or General Design Element (GDE) for update.  The 'cadOccId' can be null for
    create.  A valid 'cadOccId' must be passed when this operation is called with the 'RelativeStructureParentInfo'
    'complete' input set to true. If a valid 'cadOccId' is not supplied when 'complete' is set to true, this operation
    creates a new occurrence and any data associated against an existing occurrence is removed/lost.  This parameter
    depends on the 'CreateOrUpdateRelativeStructurePref' 'cadOccIdAttrName' preference for finding the existing BOM
    line.
    :var child: The child object for the occurrence creation.  Only item revision and General Design Element (GDE) are
    supported.  If the child occurrence exists, but the input child object is different than the existing child object,
    the existing child will be replaced by the input child.  Existing children are found referencing the occurrence
    found by the 'cadOccId' input.
    :var occInfo: The property information for this occurrence.
    """
    clientId: str = ''
    cadOccId: str = ''
    child: BusinessObject = None
    occInfo: RelOccInfo = None


@dataclass
class SaveBOMWindowsResponse(TcBaseObj):
    """
    Contains the response for the saveBOMWindows operation.
    
    :var serviceData: Any failures will be returned in the ServiceData list of partial errors.
    """
    serviceData: ServiceData = None


@dataclass
class BoundingBox(TcBaseObj):
    """
    Holds the boundingbox co-ordinates  information.
    
    :var xmin: BoundingBox x-coordinate min value in double
    :var ymin: BoundingBox y-coordinate min value in double
    :var zmin: BoundingBox z-coordinate min value in double
    :var xmax: BoundingBox x-coordinate max value in double
    :var ymax: BoundingBox y-coordinate max value in double
    :var zmax: BoundingBox z-coordinate max value in double
    """
    xmin: float = 0.0
    ymin: float = 0.0
    zmin: float = 0.0
    xmax: float = 0.0
    ymax: float = 0.0
    zmax: float = 0.0


@dataclass
class BoundingBoxInfo(TcBaseObj):
    """
    BoundingboxInfo contains boundingbox information and dataset to which it will be attached.This Dataset member
    corresponds to the AbsOccAttachment.overrideObject member to which to apply Bounding Box information.
    
    :var dataset: Dataset object to apply BoundingBox info to
    :var boundingBoxVector: List of xyz coordinate info for Dataset bounding
    """
    dataset: Dataset = None
    boundingBoxVector: List[BoundingBox] = ()


@dataclass
class VariantRulesOutput(TcBaseObj):
    """
    This contains output data for 'createVariantRules' operation.
    
    :var clientId: Identifier that helps the client to track the object(s) created.
    :var vrule: Refers to newly created VariantRule object.
    """
    clientId: str = ''
    vrule: VariantRule = None


@dataclass
class CommitDatasetFileInfo(TcBaseObj):
    """
    Contains basic information for a file to be uploaded to a dataset.
    
    :var dataset: The dataset object reference.
    :var createNewVersion: Flag that if true signifies a new version of the file should be created.
    :var datasetFileTicketInfos: List of dataset file ticket information.
    """
    dataset: Dataset = None
    createNewVersion: bool = False
    datasetFileTicketInfos: List[DatasetFileTicketInfo] = ()


@dataclass
class CreateOrUpdateAbsoluteStructureInfo3(TcBaseObj):
    """
    Input structure for createOrUpdateAbsoluteStructure.
    
    :var lastModifiedOfBVR: Last modified date of BOM view revision (BVR) under the input 'contextItemRev'.  This input
    is not required.  If this input date is different than the current last modified date and the
    'overwriteForLastModDate' preference is false the input will be ignored and processing will continue with the next
    input.  In this scenario, error 215033 will be returned.  If the dates are different and the
    'overwriteForLastModDate' preference is true, processing will continue with the current input.  In this scenario,
    error 215034 will be returned.
    :var contextItemRev: Item revision object reference of the context assembly to create or validate the occurrence. 
    This is a required reference input.
    :var bvrAbsOccInfo: List of absolute occurrence information for the BVR qualified overrides.
    :var arrAbsOccInfo: List of assembly arrangement information for the BVR or arrangement qualified overrides.  This
    input is not required.
    """
    lastModifiedOfBVR: datetime = None
    contextItemRev: ItemRevision = None
    bvrAbsOccInfo: List[AbsOccInfo2] = ()
    arrAbsOccInfo: List[AssemblyArrangementInfo] = ()


@dataclass
class CreateOrUpdateAbsoluteStructurePref3(TcBaseObj):
    """
    Preference structure for 'createOrUpdateAbsoluteStructure'.
    
    :var overwriteForLastModDate: Flag to check whether the BOM view revision will be updated if the input last
    modified date is different from the current last modified date of the object in Teamcenter.  If false, but the
    'CreateOrUpdateAbsoluteStructureInfo3' 'lastModifiedOfBVR' input specified is different than the set modified date,
    partial error 215033 will be returned.
    :var overridesToSynchronize: The attributes that the client is synchronizing when the 'complete' flag is true.
    These are the attribute names that the client is interested in. Any override properties not in this list will not
    be deleted when 'complete' is true.
    :var relationFilters: The relation overrides that the client is synchronizing when the 'complete' flag is true. 
    For instance if a relation filter of IMAN_reference relation name and 'DirectModel' 'relationTypeName' is specified
    and an override of type is in the existing structure but not specified in the input, then it will be deleted.
    :var cadOccIdAttrName: Identifies the BOM line attribute that is used to identify relative occurrences to update.
    """
    overwriteForLastModDate: bool = False
    overridesToSynchronize: List[str] = ()
    relationFilters: List[RelationAndTypesFilter] = ()
    cadOccIdAttrName: str = ''


@dataclass
class CreateOrUpdateAbsoluteStructureResponse2(TcBaseObj):
    """
    The response from the 'createOrUpdateAbsoluteStructure' operation.
    
    :var absOccOutput: Map of input 'clientId' to the created, updated or found absolute occurrence.
    :var asmArrangementOutput: Map of input 'clientId' to the created or updated assembly arrangement.
    :var formOutput: Map of input client id to the created form.
    :var datasetOutput: Information for the created or updated dataset.
    :var serviceData: The 'ServiceData'.  This operation will populate the 'ServiceData' with created or updated
    occurrences, arrangements, datasets or forms. Created forms are added as plain objects.
    """
    absOccOutput: ClientIdToAbsOccMap2 = None
    asmArrangementOutput: ClientIdToAsmArrangementMap = None
    formOutput: ClientIdToFormMap = None
    datasetOutput: List[DatasetOutput] = ()
    serviceData: ServiceData = None


@dataclass
class CreateOrUpdateRelativeStructureInfo3(TcBaseObj):
    """
    Contains lastModifiedOfBVR,a parent ItemRevision object, list of type RelativeStructureChildInfo and a  boolean
    value to specify BVR precision.
    
    :var lastModifiedOfBVR: Last Modified Date of BVR.  Used for comparison with current last modified date for
    overwrite determination.
    :var parent: Object reference of the context assembly for create or update of the child occurrence, required input
    reference.
    :var childInfo: List of type RelativeStructureChildInfo
    :var precise: Flag for updating the BVR to precise(true)/imprecise(false)
    """
    lastModifiedOfBVR: datetime = None
    parent: BusinessObject = None
    childInfo: List[RelativeStructureChildInfo2] = ()
    precise: bool = False


@dataclass
class CreateOrUpdateRelativeStructurePref3(TcBaseObj):
    """
    Contains overwriteForLastModDate, continueOnError,cadOccIdAttrName and a list of object types.
    
    :var overwriteForLastModDate: Flag to check whether BVR needs to be modified, if input last modified date is
    different from actual.
    :var continueOnError: Flag to continue with process on encountering error.
    :var cadOccIdAttrName: String representing the occurrence note type which holds the value for the CAD occurrence id
    or PSOccurrenceThread uid
    :var objectTypes: List of object types that the client is interested in, such that if the overall structure in
    Teamcenter contains object types or subtypes not in this list, that structure will not be deleted if this operation
    is complete.
    """
    overwriteForLastModDate: bool = False
    continueOnError: bool = False
    cadOccIdAttrName: str = ''
    objectTypes: List[str] = ()


@dataclass
class CreateVariantRulesInfo(TcBaseObj):
    """
    Contains the input for creating a new VariantRule object and to be associated with a ItemRevision object.
    
    :var clientID: Identifier that helps the client to track the object(s) created.
    :var vruleName: Refers to VariantRule object name.
    :var vruleDescription: Refers to VariantRule object description.
    :var rev: The ItemRevision object on which the VariantRule object to be created and attached.
    :var parent: The parent object tag to which the VariantRule object is associated. This is optional and if NULL,
    VariantRule object will be associated with ItemRevision object.
    :var relation: The relation used to associate the VariantRule object to ItemRevision object or parent.
    :var options: Refers to a list of 'OptionsInfo' struct, which has the classic variant option details.
    """
    clientID: str = ''
    vruleName: str = ''
    vruleDescription: str = ''
    rev: ItemRevision = None
    parent: BusinessObject = None
    relation: str = ''
    options: List[OptionsInfo] = ()


@dataclass
class CreateVariantRulesResponse(TcBaseObj):
    """
    This contains response for 'createVariantRules' operation.
    
    :var serviceData: The SOA framework object containing objects that were created, deleted, or updated by the service
    and error information if any.
    :var output: Refers to a list of 'VariantRulesOutput' struct objects, which in turn refers to newly created
    VariantRule objects.
    """
    serviceData: ServiceData = None
    output: List[VariantRulesOutput] = ()


@dataclass
class DataOverrideInfo(TcBaseObj):
    """
    Contains overrideData and GRM override information.
    
    :var overrideData: The AbsOccData object, which has information about qualifier, absolute ocuurence and override
    information.
    :var expandedOverrides: The GRM override information if the AbsOccData is of type AbsOccGRMAnchor.
    """
    overrideData: AbsOccData = None
    expandedOverrides: List[AbsOccDataGRMExpansionInfo] = ()


@dataclass
class DatasetFileInfo(TcBaseObj):
    """
    Contains clientId, fileName, namedReferencedName, boolean flags isText, allowReplace, and boundingBoxesAvailable as
    well as list of BoundingBoxes.
    
    :var clientId: A unique string supplied by the caller.  This ID is used to identify return data elements and
    partial errors associated with this input structure.  If 'clientId' is not provided then it can be difficult to
    align the input with any returned errors.
    :var fileName: The name of file to be uploaded.  Only the file name should be specified; the path should not be
    included.
    :var namedReferencedName: The name of the reference to use from the dataset to this file.  This input is required.
    :var isText: Flag that if true signifies the file is a text file.
    :var allowReplace: Flag that if true signifies the file can be overwritten.
    :var boundingBoxesAvailable: Flag that if true signifies bounding box information is available.
    :var boundingBoxes: List of bounding boxes, which hold the bounding box coordinate information.
    """
    clientId: str = ''
    fileName: str = ''
    namedReferencedName: str = ''
    isText: bool = False
    allowReplace: bool = False
    boundingBoxesAvailable: bool = False
    boundingBoxes: List[BoundingBox] = ()


@dataclass
class DatasetFileTicketInfo(TcBaseObj):
    """
    Contains basic information for a file to be uploaded to a dataset.
    
    :var datasetFileInfo: Member of type DatasetFileInfo.
    :var ticket: FMS ticket to use in file upload.
    """
    datasetFileInfo: DatasetFileInfo = None
    ticket: str = ''


@dataclass
class DatasetInfo(TcBaseObj):
    """
    Contains clientId, Dataset object reference for update, Name attribute value, basisName, description, Type
    attribute value, lastModifiedOfDataset, ID attribute value, datasetRev, createNewVersion flag,
    namedReferencePreference, dataList, datasetFileInfos, and List of NamedReferenceObjectInfos.
    
    :var clientId: A unique string supplied by the caller.  This ID is used to identify return data elements and
    partial errors associated with this input structure.  If 'clientId' is not provided then it can be difficult to
    align the input with any returned errors.
    :var dataset: The dataset object reference.  This input can be null for create, but it is required for update.
    :var namedReferencePreference: The preference name which holds the list of named references to delete from one
    dataset version to the next.
    :var dataList: The attribute name and values to set on the dataset.
    :var datasetFileInfos: List of dataset file information.
    :var namedReferenceObjectInfos: List of dataset named reference information.
    :var name: The dataset name attribute value.
    :var basisName: The basis name attribute value.  This is used when the name is null or blank.
    :var description: The dataset description attribute value.
    :var type: The dataset type name.
    :var lastModifiedOfDataset: Last modified date of the dataset.
    :var id: The dataset ID attribute value.
    :var datasetRev: The dataset revision attribute value.
    :var createNewVersion: Flag that if true signifies a new version of the dataset should be created.
    """
    clientId: str = ''
    dataset: Dataset = None
    namedReferencePreference: str = ''
    dataList: List[NameValueStruct] = ()
    datasetFileInfos: List[DatasetFileInfo] = ()
    namedReferenceObjectInfos: List[NamedReferenceObjectInfo] = ()
    name: str = ''
    basisName: str = ''
    description: str = ''
    type: str = ''
    lastModifiedOfDataset: datetime = None
    id: str = ''
    datasetRev: str = ''
    createNewVersion: bool = False


@dataclass
class DatasetOutput(TcBaseObj):
    """
    Created or updated dataset and file upload information.
    
    :var clientId: A unique string supplied by the caller.  This ID is used to identify return data elements and
    partial errors associated with this input structure.  If 'clientId' is not provided then it can be difficult to
    align the input with any returned errors.
    :var dataset: The dataset object reference of the created or updated dataset.
    :var commitInfo: Information for a file to be uploaded to a dataset.
    """
    clientId: str = ''
    dataset: Dataset = None
    commitInfo: List[CommitDatasetFileInfo] = ()


@dataclass
class DeleteRelativeStructureInfo3(TcBaseObj):
    """
    Information to delete relative structures.
    
    :var lastModifiedOfBVR: Last modified date of BOM view revision under the input 'parent'.  This input is not
    required.  If this input date is different than the current last modified date and the 'overwriteForLastModDate'
    input preference is false the input will be ignored and processing will continue with the next input.  In this
    scenario, error 215033 will be returned.  If the dates are different and the 'overwriteForLastModDate' input
    preference is true, processing will continue with the current input and the BVR will be modified.  In this
    scenario, error 215034 will be returned.
    :var parent: The item revision context assembly from which the child occurrences are to be removed.  This is a
    required input.  An error will be returned if a valid parent is not specified.
    :var childInfo: List of identifiers of the relative occurrences to be deleted. This is the CAD occurrence ID or
    PSOccurrenceThread UID that uniquely identifies the occurrence under a particular context item revision.
    """
    lastModifiedOfBVR: datetime = None
    parent: BusinessObject = None
    childInfo: List[str] = ()


class AbsOccDataQualifierFilter(Enum):
    """
    Specifies the filter that needs to be applied on the output absolute occurrence data that returned in the response.
    For example an occurrence can have overrides only with qualifying BVR and overrides with both upper and qualifying
    BVR. Legal values are:
    None -- When the upperQualifer is not specified all absoccdata excluding ones with upper qualifiers are returned.
    IncludeBvrOnlyQualifyingOverrides -- When an upperQualifer is specified all absoccdata that has given upper
    qualifier is returned.
    IncludeBvrAndUpperQualifyingOverrides -- Brings all absoccdata that has BVR only and which has upper and BVR
    qualifier.
    """
    None_ = 'None'
    IncludeBvrOnlyQualifyingOverrides = 'IncludeBvrOnlyQualifyingOverrides'
    IncludeBvrAndUpperQualifyingOverrides = 'IncludeBvrAndUpperQualifyingOverrides'


class NamedRefHandler(Enum):
    """
    Identifies which named references will be processed in a product structure one level expand operation.
    Legal values are : 'UseNamedRefsList, NoNamedRefs, AllNamedRefs, PreferredJT.'
    """
    UseNamedRefsList = 'UseNamedRefsList'
    NoNamedRefs = 'NoNamedRefs'
    AllNamedRefs = 'AllNamedRefs'
    PreferredJT = 'PreferredJT'


class BOMLineFilter2(Enum):
    """
    Identifies which BOM lines are to be processed. Legal values are : 'None2, ExcludeOverridden2, ExcludeICHistory2,
    ExcludeGDEs2, ExcludeImanItemLines2'.
    """
    None2 = 'None2'
    ExcludeOverridden2 = 'ExcludeOverridden2'
    ExcludeICHistory2 = 'ExcludeICHistory2'
    ExcludeGDEs2 = 'ExcludeGDEs2'
    ExcludeImanItemLines2 = 'ExcludeImanItemLines2'


"""
ClientIdToAbsOccMap2
"""
ClientIdToAbsOccMap2 = Dict[str, AbsOccurrence]


"""
ClientIdToAsmArrangementMap
"""
ClientIdToAsmArrangementMap = Dict[str, List[AssemblyArrangement]]


"""
ClientIdToFormMap
"""
ClientIdToFormMap = Dict[str, Form]



from __future__ import annotations

from tcsoa.gen.Internal.AWS2._2020_12.DataManagement import ObjectLocalizedPropertiesInfo, GetLocalizedPropertiesInput, GetLocalizedPropertiesResponse
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class DataManagementService(TcService):

    @classmethod
    def setLocalizedProperties(cls, input: List[ObjectLocalizedPropertiesInfo]) -> ServiceData:
        """
        This operation sets the values for multiple properties on multiple objects in different locales. With the
        display values capability, each localized string property can have different language translations.
        
        Use cases:
        When running Active workspace, user wants to set or modify the localized single or multiple property value
        which displayed in localization panel. This operation can be used to fulfill this requirement. By providing the
        desired business object, internal name of the properties, and specific locale name(s), local value associated
        to the locale and the localization status, this operation can be used to set multiple properties localization
        value(s) with alternative status.
        
        It should be noted that this operation is only used to set the secondary (not the master) values of the
        localized properties. User cannot package the master in the ObjectLocalizedPropertiesInfo structure.
        """
        return cls.execute_soa_method(
            method_name='setLocalizedProperties',
            library='Internal-AWS2',
            service_date='2020_12',
            service_name='DataManagement',
            params={'input': input},
            response_cls=ServiceData,
        )

    @classmethod
    def getLocalizedProperties(cls, input: GetLocalizedPropertiesInput) -> GetLocalizedPropertiesResponse:
        """
        This operation returns desired property values for any of the supported locales of the Teamcenter server.
        String and string array type properties are supported and may be localized with values for each supported
        locale, this operation will return the translated values for the input locales. If no locales are input, it
        will return translations for all supported locales.
        And it will also return the full set of translation statuses: their enumeration, localized display name and
        description for the locale in client session.
        
        Use cases:
        Retrieve the localized values for localizable properties
        
        When running Active workspace, a user may want to see the localized property value to be displayed in
        localization panel. This operation can be used to fulfill this requirement. By providing the desired business
        object, internal name of the properties, and specific locale name(s), this operation will return the localized
        properties value(s) in that particular locale(s) and the status corresponding to localized value(s) in that
        locale(s).
        
        This operation can return the fullTranslationStatuses for all the supported translation statuses for the locale
        in client session.
        """
        return cls.execute_soa_method(
            method_name='getLocalizedProperties',
            library='Internal-AWS2',
            service_date='2020_12',
            service_name='DataManagement',
            params={'input': input},
            response_cls=GetLocalizedPropertiesResponse,
        )



from tcsoa.gen.Internal_Search._2020_12.services import SearchFolderService as imp0
from tcsoa.base import TcService


class SearchFolderService(TcService):
    createOrEditSearchFolders = imp0.createOrEditSearchFolders
    exportSearchFolders = imp0.exportSearchFolders
    getSearchFolderAccessors = imp0.getSearchFolderAccessors
    getTranslatedReportSearchRecipe = imp0.getTranslatedReportSearchRecipe
    importSearchFolder = imp0.importSearchFolder



from __future__ import annotations

from tcsoa.gen.Manufacturing._2014_12.StructureSearch import AdditionalInfo, KeyValuePair
from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetSearchCrieriaFromRecipeResp(TcBaseObj):
    """
    GetSearchCriteriaFromRecipeResp structure contains the details of search critieria necessary for resolving
    Engineering BOM (EBOM) lines under the MBOM node with the recipe.
    
    :var criteriaElements: A list of SearchCriteriaElement elements, each of which captures the output data for a
    Mfg0MEMBOMRecipe object in the input list. The size of this list matches that of the input recipes.
    :var additionalInfo: A list of AdditionalInfo structures. Currently, not used.
    :var serviceData: Service data capturing partial errors using the input list index as client id.
    """
    criteriaElements: List[SearchCriteriaElement] = ()
    additionalInfo: AdditionalInfo = None
    serviceData: ServiceData = None


@dataclass
class SearchCriteriaElement(TcBaseObj):
    """
    SearchCriteriaElement structure contains the details of search critieria for a single Mfg0MEMBOMRecipe object in
    the input recipes.
    
    :var featureType: The feature type. If not empty, relatedObjs list will have the list of
    SearchCriteriaRelatedObject elements. Currently supported values are: "General", "WeldPoint", "DatumPoint", and
    "ArcWeld".
    :var logicalOperator: The logical operator associated with the feature type. Currently only "AND" and "OR" are
    supported.
    :var relatedObjs: The details of the connected objects for which the feature is being queried.
    :var searchCriteria: A list of KeyValuePair elements representing search criteria names and values.
    """
    featureType: str = ''
    logicalOperator: str = ''
    relatedObjs: List[SearchCriteriaRelatedObject] = ()
    searchCriteria: List[KeyValuePair] = ()


@dataclass
class SearchCriteriaRelatedObject(TcBaseObj):
    """
    SearchCriteriaRelatedObject structure contains the details of connected objects.
    
    :var conditionType: Indicate how the parts are connected to the feature. Currently only supported value is
    "Connected to".
    :var connectedObjs: The connected ItemRevision objects for which the feature is being queried.
    :var matchType: Currently four match types are supported: FullMatch (all parts are connected to the feature),
    PartialMatch (any of the parts is connected to the feature), NoMatch (none of the parts is connected to the
    feature), and NotApplicable (used in cases where match type is ignored).
    :var withHierarchy: If true, indicate traversal below the connectedObjs.
    """
    conditionType: str = ''
    connectedObjs: List[BusinessObject] = ()
    matchType: str = ''
    withHierarchy: bool = False



from __future__ import annotations

from tcsoa.gen.StructureManagement._2012_02.StructureVerification import AssignmentTypeDetailElement, PartialMatchCriteria
from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ExtensionAssignmentTypeDetails(TcBaseObj):
    """
    Structure that keeps assignment types details for each extension name.
    
    :var extensionName: Extension name that was compared.
    :var assignmentTypesDetails: A list of details for each assignment type.
    """
    extensionName: str = ''
    assignmentTypesDetails: List[AssignmentTypeDetailElement] = ()


@dataclass
class ExtensionComparisonSummary(TcBaseObj):
    """
    an element to capture the result of an extension comparison
    
    :var extensionName: Extension name that was compared.
    :var isDifferent: True if there is a difference in comparison of this equivalent set for this extension, false
    otherwise.
    """
    extensionName: str = ''
    isDifferent: bool = False


@dataclass
class GetAssignmentComparisonDetailsResponse(TcBaseObj):
    """
    response of method getAssignmentComparisonDetails - a vector of AssignmentTypeDetail element and serviceData
    
    :var serviceData: Object that captures any partial errors.
    :var details: The list of AssignmentType detail elements - one for each input equivalent set.
    """
    serviceData: ServiceData = None
    details: List[AssignmentTypeDetail] = ()


@dataclass
class GetComparisonSummariesResponse(TcBaseObj):
    """
    response of method getComparisonSummaries - a vector of ComparisonSummaries element and serviceData
    
    :var comparisonSummaries: The list of extension summaries elements - one for each input equivalent set.
    :var serviceData: Object that captures any partial errors.
    """
    comparisonSummaries: List[ComparisonSummaries] = ()
    serviceData: ServiceData = None


@dataclass
class AssignmentTypeDetail(TcBaseObj):
    """
    a structure to pair the AssignmentTypeDetail Element with an index into the input vector of equivalent sets of
    objects.
    
    :var index: Index of equivalent set in the input vector for which these details were calculated.
    :var equivalentLines: The list of all equivalent lines in the input equivalent set (all equivalent sources in
    sequence and then all targets in sequence).
    :var assignmentDetails: Assignment type details of this equivalent set.
    """
    index: int = 0
    equivalentLines: List[BusinessObject] = ()
    assignmentDetails: List[ExtensionAssignmentTypeDetails] = ()


@dataclass
class StringToPartialMatchCriteria(TcBaseObj):
    """
    Holds PartialMatchCriteria object for each extension name.
    
    :var extensionName: Extension name that needs to be compared. 
    An extension name is the ID of a "serverExtension" registered in Accountability Check&rsquo;s plugin.xml. It can be
    located in com.teamcenter.rac.cme.accountabilitycheck.relations package.
    :var extensionCriteria: The comparison criteria of this extension.
    """
    extensionName: str = ''
    extensionCriteria: PartialMatchCriteria = None


@dataclass
class ComparisonSummaries(TcBaseObj):
    """
    the  ExtensionComparisonSummary for each extension per input vector element
    
    :var index: Index of equivalent set in the input vector for which these results were calculated.
    :var summaries: The list of elements that capture the result of an extension comparison.
    """
    index: int = 0
    summaries: List[ExtensionComparisonSummary] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine, ImanRelation
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class TraceabilityMatrixInfo(TcBaseObj):
    """
    'TraceabilityMatrixInfo' structure represents the parameters required to create the traceability matrix between
    source and target BOM structure.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure.
    :var sourceLine: The source BOMLine object, required to create a traceability matrix.
    :var targetLine: The target BOMLine object, required to create a traceability matrix.
    """
    clientId: str = ''
    sourceLine: BOMLine = None
    targetLine: BOMLine = None


@dataclass
class TraceabilityMatrixOutput(TcBaseObj):
    """
    'TraceabilityMatrixOutput' structure represents the output parameters as a result of creating the traceability
    matrix between source and target structure. This structure contains a map object that will contain source object
    for link creation as a key of the map and list of all object as a values.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure.
    :var traceabilityMap: The TraceabilityMatrixMap object, that contains the source BOMLine object as a key and list
    of TraceabilityMatrixStructure as value in the map. This map will be parsed on the client to show proper link count
    for specific source and target object in the traceability matrix.
    """
    clientId: str = ''
    traceabilityMap: TraceabilityMatrixMap = None


@dataclass
class TraceabilityMatrixResponse(TcBaseObj):
    """
    'TraceabilityMatrixResponse' structure represents list of TraceabilityMatrixOutput structure containing the
    TraceabilityMatrixMap map object along with the ServiceData.
    
    :var serviceData: The Service Data.
    :var output: A list of TraceabilityMatrixOutput structure that hold the information to generate traceability matrix
    selected source and target objects. This information will be parsed on the client to show the proper data in the
    matrix.
    """
    serviceData: ServiceData = None
    output: List[TraceabilityMatrixOutput] = ()


@dataclass
class TraceabilityMatrixStructure(TcBaseObj):
    """
    'TraceabilityMatrixStructure' structure represents the output parameters to specify that for specified source
    BOMLine and specified target BOMLine object these TraceLink relation objects are created. This structure contains
    the list of all TraceLink relation between specified source BOMLine and each target BOMLine object.
    
    :var bomLine: A target BOMLine object with respect to each source BOMLine from the source structure.
    :var relations: A list of TraceLink relation object that defined that specified each source and target BOMLine
    object.
    """
    bomLine: BOMLine = None
    relations: List[ImanRelation] = ()


"""
This is a map that will contain the source BOMLine object as a key and 'TraceabilityMatrixStructure' structure vector as values.
"""
TraceabilityMatrixMap = Dict[BOMLine, List[TraceabilityMatrixStructure]]



from __future__ import annotations

from tcsoa.gen.ProjectManagement._2007_12.ScheduleManagement import DemandProfileRequest, DemandProfileResponses
from typing import List
from tcsoa.base import TcService


class ScheduleManagementService(TcService):

    @classmethod
    def getDemandProfile(cls, requests: List[DemandProfileRequest]) -> DemandProfileResponses:
        """
        Calculates the demand profile data for a schedule based on the initial
        input request to the Application Interface..
        """
        return cls.execute_soa_method(
            method_name='getDemandProfile',
            library='ProjectManagement',
            service_date='2007_12',
            service_name='ScheduleManagement',
            params={'requests': requests},
            response_cls=DemandProfileResponses,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMWindow, WorkspaceObject
from tcsoa.gen.Internal.StructureManagement._2008_06.Structure import CopyRecursivelyResponse, FindHighestFindNumInExpandInput, FindHighestFindNumberInExpandResponse
from typing import List
from tcsoa.base import TcService


class StructureService(TcService):

    @classmethod
    def copyRecursively(cls, objectToClone: WorkspaceObject, copyActionRulesKey: str, templateBOMWindow: BOMWindow, windowOfSelectedLine: BOMWindow, newName: str, newDescription: str, newId: str, newRevId: str, copyFutureEffectivities: bool) -> CopyRecursivelyResponse:
        """
        copyRecursively - perform cloning operation based on the template provide
        """
        return cls.execute_soa_method(
            method_name='copyRecursively',
            library='Internal-StructureManagement',
            service_date='2008_06',
            service_name='Structure',
            params={'objectToClone': objectToClone, 'copyActionRulesKey': copyActionRulesKey, 'templateBOMWindow': templateBOMWindow, 'windowOfSelectedLine': windowOfSelectedLine, 'newName': newName, 'newDescription': newDescription, 'newId': newId, 'newRevId': newRevId, 'copyFutureEffectivities': copyFutureEffectivities},
            response_cls=CopyRecursivelyResponse,
        )

    @classmethod
    def findHighestFindNumberInExpand(cls, input: List[FindHighestFindNumInExpandInput]) -> FindHighestFindNumberInExpandResponse:
        """
        Can be used to find the highest sequence number in the result of an expand, without actually returning all the
        objects in the expand.
        """
        return cls.execute_soa_method(
            method_name='findHighestFindNumberInExpand',
            library='Internal-StructureManagement',
            service_date='2008_06',
            service_name='Structure',
            params={'input': input},
            response_cls=FindHighestFindNumberInExpandResponse,
        )



from __future__ import annotations

from tcsoa.gen.Manufacturing._2013_12.Model import AppearancePathInput, ComputeAppearancePathResponse
from tcsoa.gen.Manufacturing._2013_12.ResourceManagement import GetVendorCatalogInfoResponse, ImportVendorCatalogHierarchyResponse, GetStepP21FileCountsResponse, ImportStepP21FilesResponse
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class ResourceManagementService(TcService):

    @classmethod
    def getStepP21FileCounts(cls, classIDs: List[str]) -> GetStepP21FileCountsResponse:
        """
        This operation retrieves the count of STEP P21 product files that are available in a GTC vendor catalog in and
        below the specified classes.
        """
        return cls.execute_soa_method(
            method_name='getStepP21FileCounts',
            library='Manufacturing',
            service_date='2013_12',
            service_name='ResourceManagement',
            params={'classIDs': classIDs},
            response_cls=GetStepP21FileCountsResponse,
        )

    @classmethod
    def getVendorCatalogInfo(cls) -> GetVendorCatalogInfoResponse:
        """
        This operation retrieves information about the GTC vendor catalogs that are available on the Teamcenter server
        machine. The multi-value preference "MRMGTCVendorCatalogRootDir" allows you to specify one or multiple root
        directories where those catalogs can be stored. This operation scans the given directories for GTC vendor
        catalogs and returns detailed information for each catalog.
        """
        return cls.execute_soa_method(
            method_name='getVendorCatalogInfo',
            library='Manufacturing',
            service_date='2013_12',
            service_name='ResourceManagement',
            params={},
            response_cls=GetVendorCatalogInfoResponse,
        )

    @classmethod
    def importStep3DModels(cls, icoIDs: List[str]) -> ServiceData:
        """
        This operation imports STEP 3D model files for Generic Tool Catalog (GTC) vendor catalog components. The STEP
        files are converted to NX part and to JT file format and imported into Teamcenter. If needed, items are created
        for the Classification objects (ICO). One UGMaster dataset for the NX part file and one DirectModel dataset for
        the JT file is created below the (new) item.
        
        Use cases:
        There are two different use cases:
            A) The specified ICO is classified in a GTC vendor catalog class
        (the ICO has an attribute -159003 "3D Model file name")
            B) The specified ICO is classified in an MRL MyComponents class
        (the ICO does not have an attribute -159003 "3D Model file name")
        
        In use case A, this operation retrieves the file name of the 3D model directly from GTC attribute 
        -159003 "3D Model file name".
        (Another attribute ID instead of -159003 can be defined in the "MRMGTC3DModelAttributeID" preference.)
        
        In use case B, the SOA operation checks if there is an Manufacturing Resource Libraray (MRL) attribute -40930
        "Vendor Reference Object ID" in the given ICO. This attribute is used to store the reference from the MRL
        MyComponents ICO to the GTC vendor catalog ICO. If this attribute exists in the ICO's class and has a valid ICO
        ID as value, the 3D model file name will be retrieved from the referenced ICO. (see use case A)
        (Another attribute ID instead of -40930 can be defined in the "MRMGTCReferenceObjectAttributeID"preference.)
        
        The SOA operation retrieves the information about the server-sided directory, where all STEP 3D models of the
        catalog are stored, based on the class of the GTC vendor catalog ICO.
        
        If there is a problem during importing one of the 3D models, the operation will continue importing the 3D
        models of the following ICOs.
        
        Note: The Graphics Builder has to be configured properly for this operation to work.
        """
        return cls.execute_soa_method(
            method_name='importStep3DModels',
            library='Manufacturing',
            service_date='2013_12',
            service_name='ResourceManagement',
            params={'icoIDs': icoIDs},
            response_cls=ServiceData,
        )

    @classmethod
    def importStepP21Files(cls, classID: str, importOptions: int) -> ImportStepP21FilesResponse:
        """
        This operation imports STEP P21 files containing the attributes values of manufacturing components (vendor
        product data) into the vendor catalog Classification classes. 
        It creates Classification objects (ICOs) and associated data. Depending on the available data in the input
        directory, the associated items might be created to store drawing files and/or images.
        """
        return cls.execute_soa_method(
            method_name='importStepP21Files',
            library='Manufacturing',
            service_date='2013_12',
            service_name='ResourceManagement',
            params={'classID': classID, 'importOptions': importOptions},
            response_cls=ImportStepP21FilesResponse,
        )

    @classmethod
    def importVendorCatalogHierarchy(cls, vendorCatalogRootDir: str, importOptions: int) -> ImportVendorCatalogHierarchyResponse:
        """
        This operation imports the Classification class hierarchy (including class icons and images) of a GTC (Generic
        Tool Catalog) vendor catalog. The catalog hierarchy will be imported below the Manufacturing Resource Library
        (MRL) "Vendor Catalogs" class. The vendor catalog root directory that is needed as input parameter can be
        retrieved using the service operation getVendorCatalogInfo().
        """
        return cls.execute_soa_method(
            method_name='importVendorCatalogHierarchy',
            library='Manufacturing',
            service_date='2013_12',
            service_name='ResourceManagement',
            params={'vendorCatalogRootDir': vendorCatalogRootDir, 'importOptions': importOptions},
            response_cls=ImportVendorCatalogHierarchyResponse,
        )

    @classmethod
    def updateNXToolAssemblies(cls, icoIDs: List[str], identifyCutAndNoCut: bool, generateSpinning: bool, setToolJunctions: bool, writePartAttributes: bool) -> ServiceData:
        """
        This operation triggers specific actions on the NX side to update one or mutliple tool assemblies for use in NX
        CAM. The tool assemblies are identified by their Classification object (ICO) IDs. Initially, the tool
        assemblies are created in the Resource Manager in Teamcenter. Then this operation uses the Graphics Builder and
        calls some NX user functions. UGMaster datasets and an NX part files are created for the tool assemblies. The
        different input parameters trigger more actions.  To ensure that this operation works properly, the "NX Tool
        Type" (MRL attribute -45110) has to be defined in the tool assemblies. Turning tool assemblies have to have a
        "Tracking Point" (MRL attribute -45015) specified. This operation works only for tool assemblies that are
        classified using the Manufacturing Resource Library (MRL).
        Note: The Graphics Builder has to be configured properly for this operation to work.
        """
        return cls.execute_soa_method(
            method_name='updateNXToolAssemblies',
            library='Manufacturing',
            service_date='2013_12',
            service_name='ResourceManagement',
            params={'icoIDs': icoIDs, 'identifyCutAndNoCut': identifyCutAndNoCut, 'generateSpinning': generateSpinning, 'setToolJunctions': setToolJunctions, 'writePartAttributes': writePartAttributes},
            response_cls=ServiceData,
        )


class ModelService(TcService):

    @classmethod
    def computeAppearancePath(cls, input: AppearancePathInput) -> ComputeAppearancePathResponse:
        """
        This service computes and returns the values for APNUID and AbsOccUID propertis.
        
        APN - bl_apn_uid_in_topline_context
        AbsOccUID - bl_absocc_uid_in_topline_context
        
        Input: parent object and a list of paths.
                 Each path starts reletively from the parent object.
                   In case the parent does not have APN, the service will calculate it to the parent as well.
        Response : vector of values of the adove propeties according to each path.
        
        If the input is BOMLine (and not Fnd0BOMLineLite) there might be performance issue due to the fact that the
        modified obejct would be put in the serviceData and the properties that will be brought for this object would
        be according to the PolicyFile (in this case the best practice is to use Dymanic Policy).
        
        Fnd0BOMLineLite does not have as much properties as BOMLine and therefore even if the input is large, not many
        properties would be calculated in the Policy File.
        
        """
        return cls.execute_soa_method(
            method_name='computeAppearancePath',
            library='Manufacturing',
            service_date='2013_12',
            service_name='Model',
            params={'input': input},
            response_cls=ComputeAppearancePathResponse,
        )



from __future__ import annotations

from tcsoa.gen.Internal.AWS2._2013_12.DataManagement import GetTCSessionInfoResponse3, GetTCSessionInfoResponse
from tcsoa.gen.Internal.AWS2._2013_12.OrganizationManagement import GroupMembershipInput2
from tcsoa.gen.Internal.AWS2._2012_10.OrganizationManagement import GroupMembershipResponse
from typing import List
from tcsoa.gen.Internal.AWS2._2013_12.Workflow import TaskSearchResponse, TaskSearchInput
from tcsoa.base import TcService


class DataManagementService(TcService):

    @classmethod
    def getTCSessionInfo2(cls) -> GetTCSessionInfoResponse:
        """
        This operation gets information about the current user's Teamcenter session.s
        """
        return cls.execute_soa_method(
            method_name='getTCSessionInfo2',
            library='Internal-AWS2',
            service_date='2013_12',
            service_name='DataManagement',
            params={},
            response_cls=GetTCSessionInfoResponse,
        )

    @classmethod
    def getTCSessionInfo3(cls, extraInfoIn: List[str]) -> GetTCSessionInfoResponse3:
        """
        This operation provides information about the current user's Teamcenter session. The additional  information
        provided in getTCSessionInfo3 as compared to getTCSessionInfo2 is a map of extra information. Any new
        information that needs to be passed to the client will be put in this map.
        
        Use cases:
        After the user logs in to Active Workspace this operation is invoked by the Active Workspace client to get
        information related to the user session like Groups, Projects, and the Teamcenter Server version.
        """
        return cls.execute_soa_method(
            method_name='getTCSessionInfo3',
            library='Internal-AWS2',
            service_date='2013_12',
            service_name='DataManagement',
            params={'extraInfoIn': extraInfoIn},
            response_cls=GetTCSessionInfoResponse3,
        )


class WorkflowService(TcService):

    @classmethod
    def performTaskSearch(cls, searchInput: TaskSearchInput) -> TaskSearchResponse:
        """
        This operation searches for EPMTasks in a user's inbox based on input search criteria.
        
        Use cases:
        Use this operation to search for tasks in logged in user's inbox. User can provide multiple search criteria to
        search for tasks. For example :
        
        Search  tasks by specific task type
        User wants to search for all the Do tasks in his inbox.
        
        Search tasks by assignee
        User wants to search for all the task in his inbox which are assigned to a specific group.
        
        Search tasks by due date
        User wants to search for all the tasks  in his inbox  based on a due date. The  due date can  be specified as a
        specific date or month or year.
        """
        return cls.execute_soa_method(
            method_name='performTaskSearch',
            library='Internal-AWS2',
            service_date='2013_12',
            service_name='Workflow',
            params={'searchInput': searchInput},
            response_cls=TaskSearchResponse,
        )


class OrganizationManagementService(TcService):

    @classmethod
    def getGroupMembership2(cls, input: GroupMembershipInput2) -> GroupMembershipResponse:
        """
        This operation searches for active GroupMember based on name, role and group inputs . Literal and partial
        strings (strings with wildcard character '*') are supported. Literal string are AND together and partial
        strings are grouped OR and AND with with the literal strings.
        Using the following search critera:
        
        userId = "tc*"
        userName =  "tcadmin"
        groupName = "dba"
        roleName = "DB*"
        
        The operation will assemble the final search criteria as follows: 
        
        username = "tcadmin" AND groupName = "dba" AND ( userId = "tc*" OR roleName = "DB*" ). 
        
        Following examples demonstrate how this operation can be used to get GroupMember objects based on different
        inputs:
        
        - Search for GroupMember objects which  have User objects where userd_id, user_name contain  "XYZ". Input for
        this search will be :
        
        
        userId   - "*XYZ*"
        userName - "*XYZ*"
        
        - Search for GroupMember objects which have User or Role or Group objects where userd_id, user_name,
        role_name,or name contain  "xyz". Input for this search will be :
        
        
        userId      -   "*xyz*"
        userName    -  "*xyz*"
        roleName    -  "*xyz*"
        groupName   -  "*xyz*"
        
        - Search for GroupMember objects which have User or Role objects where userd_id,user_name, role_name contain 
        "xyz" and has Group object with name "dba". Inputs for this search will be :
        
        
        userId     -   "*xyz*"
        userName   -  "*xyz*"
        roleName   -  "*xyz*"
        groupName  -  "dba"
        
        - Search for GroupMember objects which have User or Group objects where userd_id, user_name, name contain 
        "xyz" and has Role object with role_name  "DBA". Inputs for this search will be :
        
        
        userId           -   "*xyz*"
        userName   -  "*xyz*"
        roleName   -  "DBA"
        groupName  -  "*xyz*"   
        
        - Search for GroupMember objects which have User objects where user_id, user_name contain"xyz" and Role object
        with role_name  "DBA" and Group object with name "dba". Inputs for this search will be :
        
        
        userId     -   "*xyz*"
        userName   -  "*xyz*"
        roleName   -  "DBA"
        groupName  -  "dba"
        
        - Search for GroupMember objects which have User objects where user_id "xyz" and user_name "xyz" and Role
        object with role_name  "DBA" and Group object with name "dba". Inputs for this search will be :
        
        
        userId     -   "xyz"
        userName   -  "xyz"
        roleName   -  "DBA"
        groupName  -  "dba"
        
        Use cases:
        Search GroupMember(s) againt signoff profile of a select-signoff task:
        
        User has an active select-signoff-task with signoff profile. Profile specifies the group, role and number of
        the users who are allowed to perform the review. This task has following profiles:
        - * / DBA / 1 ( Any User  who has a role "DBA")
        - dba / * / 1 ( Any User who is in  group "dba" )
        - dba / DBA / 1 ( Any User who has a role "DBA" and in group "dba" )
        - Engineering++ / * / 1 ( Any  User from "Engineering" sub-group )
        - GroupA++ / RoleA / 1 ( Any User who has a role "RoleA" and from "GroupA" sub-group )
        
        
        
        Here user wants to search for all the active GroupMembers in Teamcenter organization with above profiles to
        signoff the task.
        """
        return cls.execute_soa_method(
            method_name='getGroupMembership2',
            library='Internal-AWS2',
            service_date='2013_12',
            service_name='OrganizationManagement',
            params={'input': input},
            response_cls=GroupMembershipResponse,
        )



from __future__ import annotations

from tcsoa.gen.Internal.ChangeManagement._2020_01.MassUpdate import SaveImpactedAssembliesIn, HasActiveMarkupAssociatedOut
from tcsoa.gen.BusinessObjects import ItemRevision
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class MassUpdateService(TcService):

    @classmethod
    def hasActiveMarkupAssociated(cls, changeObject: ItemRevision) -> HasActiveMarkupAssociatedOut:
        """
        This operation checks if there are active Mass Update Fnd0Markup objects associated with the PSBOMViewRevision
        of impacted ItemRevision objects related to the input change object ItemRevision.
        
        Use cases:
        When you remove the problem object ItemRevision from the change object ItemRevision of a Mass Update, any
        associated Fnd0Markup objects will be removed. Before the removal of the Fnd0Markup objects, this operation
        will check for the existence of associated Fnd0Markup objects and a confirmation message will be displayed.
        """
        return cls.execute_soa_method(
            method_name='hasActiveMarkupAssociated',
            library='Internal-ChangeManagement',
            service_date='2020_01',
            service_name='MassUpdate',
            params={'changeObject': changeObject},
            response_cls=HasActiveMarkupAssociatedOut,
        )

    @classmethod
    def saveImpactedAssemblies(cls, changeObject: ItemRevision, impactedObjectsInfo: List[SaveImpactedAssembliesIn]) -> ServiceData:
        """
        This operation saves proposed changes as Fnd0MarkupChange objects for an Fnd0Markup related to the
        PSBOMViewRevision of an impacted ItemRevision. Based on the modified properties specified on
        Fnd0MUImpactedParents, the operation will add, update or delete markup changes.
        
        Use cases:
        When you attach a problem ItemRevision to the ChangeItemRevision, corresponding impacted assembly parents of
        the problem object are listed for the Mass Update.
        You can select an action (Add Part, Replace Part, Remove Part, Remove Part as Substitute, and Add Part as
        Substitute) and choose a corresponding proposed solution object. When you save the action including its
        solution object, this operation stores it as Fnd0MarkupChange objects within the Fnd0Markup object related to
        the PSBOMViewRevision of the impacted ItemRevision.
        """
        return cls.execute_soa_method(
            method_name='saveImpactedAssemblies',
            library='Internal-ChangeManagement',
            service_date='2020_01',
            service_name='MassUpdate',
            params={'changeObject': changeObject, 'impactedObjectsInfo': impactedObjectsInfo},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from typing import Dict, List
from tcsoa.gen.BusinessObjects import WorkspaceObject, POM_object


@dataclass
class SetChangeContextResponse(TcBaseObj):
    """
    Contains the input edit context and the configured input objects.
    
    :var serviceData: Contains any partial errors that occur during the operation.
    :var configuredObjects: Contains the input edit context and the configured input objects.
    """
    serviceData: ServiceData = None
    configuredObjects: ChangeContextMap = None


"""
Maps the context used to configure objects and the list of configured objects.
"""
ChangeContextMap = Dict[WorkspaceObject, List[POM_object]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Internal.AWS2._2016_04.DataManagement import GetInitialTableRowDataResponse, GetStyleSheetResponse, LoadDataForEditingResponse, GetStyleSheetIn, SaveEditAndSubmitInfo, LoadDataForEditingInfo
from typing import List
from tcsoa.gen.Internal.AWS2._2012_10.DataManagement import SaveEditAndSubmitResponse
from tcsoa.base import TcService


class DataManagementService(TcService):

    @classmethod
    def getStyleSheet(cls, processEntireXRT: bool, input: List[GetStyleSheetIn]) -> GetStyleSheetResponse:
        """
        This operation returns the stylesheets and the required data to present that stylesheet for each input object.
        
        Use cases:
        Use Case 1: Open an object in ActiveWorkspace.
        When an object is selected in ActiveWorkspace, the user may choose to invoke the open operation. When that
        operation is executed the getStylesheet call is invoked and the Stylesheet is processed in order to display the
        data to the user.
        
        Use Case 2: List view with Summary
        When an object is selected in the ActiveWorkspace navigator, when in list view with summary mode, the
        getStyleSheet SOA operation is invoked, and the returned data is parsed/processed in order to present the data
        to the user in the Summary panel.
        
        Use Case 3: Show Object Info
        When an object is selected in ActiveWorkspace, a user may choose the "Show Object Info" command.  When
        executed, a panel slides out from the right hand side of the application, and the selected objects data is
        presented.  In order to populate the panel, the getStylesheet call is invoked and the returned data is parsed
        and processed in order to build the UI.
        """
        return cls.execute_soa_method(
            method_name='getStyleSheet',
            library='Internal-AWS2',
            service_date='2016_04',
            service_name='DataManagement',
            params={'processEntireXRT': processEntireXRT, 'input': input},
            response_cls=GetStyleSheetResponse,
        )

    @classmethod
    def loadDataForEditing(cls, inputs: List[LoadDataForEditingInfo]) -> LoadDataForEditingResponse:
        """
        This SOA method ensures that the properties can be edited, and returns the last save date of the related
        objects for optimistic edit
        """
        return cls.execute_soa_method(
            method_name='loadDataForEditing',
            library='Internal-AWS2',
            service_date='2016_04',
            service_name='DataManagement',
            params={'inputs': inputs},
            response_cls=LoadDataForEditingResponse,
        )

    @classmethod
    def saveEditAndSubmitToWorkflow(cls, inputs: List[SaveEditAndSubmitInfo]) -> SaveEditAndSubmitResponse:
        """
        This operation saves the modified properties for the given input objects and submits the objects to a workflow.
        The workflow is submitted only if all of the save operations are successful. If the save fails for a single
        object  none of  the input objects will be submitted to a workflow.
        
        Use cases:
        User can modify the object(s) properties and submit the object(s) to workflow in one operation. This operation
        first saves the modified properties and then initiates the workflow process for all input objects.
        """
        return cls.execute_soa_method(
            method_name='saveEditAndSubmitToWorkflow',
            library='Internal-AWS2',
            service_date='2016_04',
            service_name='DataManagement',
            params={'inputs': inputs},
            response_cls=SaveEditAndSubmitResponse,
        )

    @classmethod
    def getInitialTableRowData(cls, owningObject: BusinessObject, tablePropertyName: str) -> GetInitialTableRowDataResponse:
        """
        This operation returns the proposed values to act as initial values when a new row object needs to be added to
        the table. User can then do further modification as needed, and perform save operation to create the row object
        in the database.
        
        Use cases:
        When user clicks on the add button for table property, client can add a temporary row showing some of the
        system generated properties populated. Property value generation will honor related LOVs, naming rules, default
        values etc. The newly added row should behave as if user has added a new row. User can then perform save to
        make the row persist in the database.
        
        Exceptions:
        >141156  An error has occurred while creating initial row values of type tablePropertyName. Please refer to the
        Teamcenter server syslog file for more information.
        """
        return cls.execute_soa_method(
            method_name='getInitialTableRowData',
            library='Internal-AWS2',
            service_date='2016_04',
            service_name='DataManagement',
            params={'owningObject': owningObject, 'tablePropertyName': tablePropertyName},
            response_cls=GetInitialTableRowDataResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, Effectivity, Awb0SavedBookmark, Awb0Element, Awb0ProductContextInfo
from tcsoa.gen.Internal.ActiveWorkspaceBom._2015_07.OccurrenceManagement import SearchSortCriteria2, Criteria
from typing import Dict, List
from tcsoa.gen.Internal.ActiveWorkspaceBom._2012_10.OccurrenceManagement import ProductInfoData
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ReplaceElementResponse(TcBaseObj):
    """
    'ReplaceElementResponse' contains a list of 'replacable' Awb0Element if dry run is specified.
    
    :var replacableElements: List of Awb0Element which can be replaced with given 
    'replacement'.
    :var columnConfig: Effective column configuration for the client scope URI.
    :var serviceData: Contains the list of updated Awb0Element(s) in updated list and 
    partial error if any.
    :var reloadContent: Indicates if content should be reloaded. Set to true if input element is replaced with element
    of different type else false.
    """
    replacableElements: List[Awb0Element] = ()
    columnConfig: ColumnConfig = None
    serviceData: ServiceData = None
    reloadContent: bool = False


@dataclass
class ReplaceInput(TcBaseObj):
    """
    This structure contains 'replacement' to be set as underlying object of the list of 'elements'.
    
    :var elements: List of Awb0Element whose underlying object to be replaced 
    with 'replacement'.
    :var replacement: The BusinessObject to be replaced for input 'elements'. It can 
    be ItemRevision or GenericDesignElement.
    :var productContextInfo: The Awb0ProductContextInfo object containing configuration 
    information.
    :var requestPref: Map of preference names and values pairs 
    (string / list of string ). Allowed preference name is dryRun. Values can be any valid string. Preference names and
    values are case sensitive.
    :var columnConfigInput: The column config input criteria to get the effective column configurations.
    """
    elements: List[Awb0Element] = ()
    replacement: BusinessObject = None
    productContextInfo: Awb0ProductContextInfo = None
    requestPref: RequestPreference4 = None
    columnConfigInput: ColumnConfigInput = None


@dataclass
class SubsetInput2(TcBaseObj):
    """
    SubsetInput2 contains list of recipe and productInfo to fetch updated recipes and filters.
    
    :var productInfo: Awb0ProductContextInfo object containing the product with its configuration information. The
    product and configuration is used to fetch subset filters.
    :var recipe: List of Criteria as held by client, this is used to update filters which are currently applied.
    :var searchSortCriteria: A list of criterion to be used for sorting the filtered results.
    :var searchFilterFieldSortType: Type of sorting that needs to be performed on the search filters. Valid values are
    "Alphabetical", and "Priority".
    :var requestPref: Map of preference names and values pairs (string / list of string ). Allowed preference name is
    selectedFilter. Values can be any valid string. Preference names and values are case sensitive.
    """
    productInfo: Awb0ProductContextInfo = None
    recipe: List[Criteria] = ()
    searchSortCriteria: List[SearchSortCriteria2] = ()
    searchFilterFieldSortType: str = ''
    requestPref: RequestPreference3 = None


@dataclass
class ColumnConfig(TcBaseObj):
    """
    This structure contains information for a column configuration within a client scope URI.  It contains a unique
    column config id, a list of column definition information, and the operation type used to finalize the columns.
    
    :var columnConfigId: The unique identifier of the column configuration.
    :var operationType: The operation that  was used to finalize the columns to be returned back.
    Supported values are:
    "Intersection", "Union" and "Configured"
    :var columnsToExclude: List of  columns which should be excluded from the final list being returned. The value
    provided should be in the format "TypeName.PropertyName". Both type name and property name should be internal
    values. 
    For example: ItemRevision.sequence_id, where '.' is the delimiter
    :var elementTypesForArrange: List of element types.
    :var columns: Ordered list of column details.
    :var hasColumnsChanged: Indicates whether table display should be udpated.
    """
    columnConfigId: str = ''
    operationType: str = ''
    columnsToExclude: List[str] = ()
    elementTypesForArrange: List[str] = ()
    columns: List[ColumnDefInfo] = ()
    hasColumnsChanged: bool = False


@dataclass
class ColumnConfigInput(TcBaseObj):
    """
    Contains input information required to retrieve UI column configurations from the Teamcenter database.
    
    :var clientName: The name of a client application, as represented by an instance of Fnd0Client in the Teamcenter
    database.  This value must match the value of fnd0ClientName property. 
    For example: The client name for Active Workspace is "AWClient"
    :var hostingClientName: Specifies the name of a hosting client application, as represented by an instance of
    Fnd0Client, in the Teamcenter databases.  This value must match a value of the fnd0ClientName property.  
    For example: If client A is integrated with client B and the user can invoke client B commands from within client
    A, the input would specify client A as hosting client and client B as the client. If the caller wanted native
    commands for client A, client A would be specified as client and hosting client would be empty.
    :var operationType: The operation that  needs to be applied to finalize the columns to be returned back.
    Supported values are:
    "Intersection" - Gets the intersection of the columns for the types found in search results.
    "Union" - Gets all the columns for the types found in search results.
    "Configured" - Gets all the columns defined for requested scope irrespective of types in the search results . If it
    does not find any configuration at the specified scope it will  search up in the order of scopes User, Role, Group
    and Site.
    :var fetchColumnConfig: Indicates whether column configuration to be fetched or not, this is for future use.
    """
    clientName: str = ''
    hostingClientName: str = ''
    operationType: str = ''
    fetchColumnConfig: bool = False


@dataclass
class ColumnDefInfo(TcBaseObj):
    """
    Contains details about a specific column. This includes the type of object for which the column is applicable, the
    name of the property displayed in the column, a flag indicating if the column should be used to order information
    displayed in the client, pixel width of the column, a flag indicating if the column should be hidden and  the
    column sort order .
    
    :var typeName: The business object type for the value displayed in the column.  This can be any valid Teamcenter
    business object type.
    :var propertyName: The property name for the value displayed in the column.
    :var pixelWidth: The pixel width for the column. Valid pixel widths are integer values between 1 and 500.
    :var columnOrder: The column order value is used to arrange the columns in order.
    :var hiddenFlag: If true,  the column should be hidden on the client user interface.
    :var sortPriority: Sort priority set on column helps identify the order in which the columns should be used during
    sort.
    Sort priority value will be zero for columns not marked for sorting.
    :var sortDirection: How the columns are sorted.  Supported values are: "Ascending" and "Descending". This value
    will be empty if the column is not marked for sorting.
    """
    typeName: str = ''
    propertyName: str = ''
    pixelWidth: int = 0
    columnOrder: int = 0
    hiddenFlag: bool = False
    sortPriority: int = 0
    sortDirection: str = ''


@dataclass
class AddToBookmarkInputData2(TcBaseObj):
    """
    The structure contains object to be added as child of input parentElement or productContext.
    
    :var productsToBeAdded: The products to be added to the Awb0AutoBookmark object associated with the input
    Awb0SavedBookmark object. The product can be of Item or ItemRevision type.
    :var bookmark: The Awb0SavedBookmark object. The products will be added to the Awb0AutoBookmark object associated
    with this object.
    :var columnConfigInput: The column config input criteria to get the effective column configurations.
    """
    productsToBeAdded: List[BusinessObject] = ()
    bookmark: Awb0SavedBookmark = None
    columnConfigInput: ColumnConfigInput = None


@dataclass
class EffectivityInput(TcBaseObj):
    """
    This structure contains 'effectivities' to be attached to the list of 'elements'.
    
    :var elements: List of Awb0Element for which effectivities to be added or 
    removed.
    :var effectivities: List of Effectivity objects which are added or removed  to/from  
    elements.
    :var addOrRemove: Indicates whether to add or remove Effectivity(s). When true, 
    Effectivity is added. Otherwise, Effectivity is removed.
    """
    elements: List[Awb0Element] = ()
    effectivities: List[Effectivity] = ()
    addOrRemove: bool = False


@dataclass
class AddToBookmarkResp2(TcBaseObj):
    """
    List of the ProductInfoData objects associated with the products that were added. The ServiceData has the modified
    Awb0SavedBookmark object.
    
    :var addedProductsInfo: The list of structures containing the information associated to the products added to the
    Awb0Autobookmark.
    :var columnConfig: Effective column configuration for the client scope URI.
    :var serviceData: The ServiceData containing modified Awb0SavedBookmark and partial error if any.
    """
    addedProductsInfo: List[ProductInfoData] = ()
    columnConfig: ColumnConfig = None
    serviceData: ServiceData = None


"""
The map which can have a key and value pair, used for occurrences expansion filters or criteria or options. The key and value are case sensitive.
"""
RequestPreference3 = Dict[str, List[str]]


"""
The map which can have a key and value pair, used to specify the mode of replace operation. The key and value are case sensitive.
"""
RequestPreference4 = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.Query._2020_04.SavedQuery import BusinessObjectQueryInput4
from typing import List
from tcsoa.gen.Query._2007_09.SavedQuery import SavedQueriesResponse
from tcsoa.base import TcService


class SavedQueryService(TcService):

    @classmethod
    def executeBOQueriesWithSortAndContext(cls, inputs: List[BusinessObjectQueryInput4]) -> SavedQueriesResponse:
        """
        Execute business object searches (Simple Search) and return sorted search results. 
        This operation is invoked to query business object instances including Logical Object instances. The input
        structure identifies the type, clauses, sorting options, and max number to return for the search. Additionally,
        a map of of property name and ConfigurationContext pairs are input to support configuration context as a filter
        for the search.
        
        Use cases:
        This operation is invoked to query business object instances including Logical Object instances.
        
        The Logical Object query use cases are described below: The properties used in search clauses and sort options
        are presented properties of  Logical Object.
        
        Logical Object Query Use Case Data:
        
        1.    Logical object definition 1
            Name                                            :     "LogicalItemRevision"
            Root business object                :      ItemRevision
            Included logical object ID1        :    "LogicalItemRevisionDomainA"
        
        2.    Logical object definition 2
            Name                                            :     "LogicalItemRevisionDomainA"
            Root business object                    :      ItemRevision
        
        
        
         Use Case 1: 
         
        BusinessObjectQueryInput4 [0] = { 
            typeName                       =      "LogicalItemRevision", 
            clauses [0]= 
                { 
                    propName = "object_name",
                    propValue = "design1",
                    mathOperator = "=",
                    logicOperator = "AND"
                  }
            sortOptions [0]= 
                { 
                    sortAttribute = "item_revision_id",
                    sortOrder = "AscendingOrder"
                }
            maxNumToReturn         =     30,
            requestId             =     1,
            clientId             =     1,
        }
        
        This operation will return instances of "LogicalItemRevision", object_name of root "Item Revision" is
        "design1". Since no configuration context is provided for "Included logical object ID1", all the instances of 
        "LogicalItemRevisionDomainA" will be returned.
        
        The input sort options will return instances of "LogicalItemRevision", sorted by item_revision_id in ascending
        order.
        
        The presented properties on the instances of "LogicalItemRevision" and its included logical object:
        "LogicalItemRevisionDomainA" will be returned through Service Data.  
        
        Use Case 2:
        
        BusinessObjectQueryInput4 [0] = { 
            typeName                       =      "LogicalItemRevision", 
            clauses [0]= 
            { 
                    propName = "object_name",
                propValue = "design1",
                mathOperator = "=",
                logicOperator = "AND"
              }
            maxNumToReturn         =     30,
            requestId             =     1,
            clientId             =     1,
            configurationContextMap =           [<"Included logical object ID1", "ConfigContextUid1">]
        }
        
        This operation will return instances of "LogicalItemRevision", object_name of root "Item Revision" is
        "design1". Since the configuration context is provided for "Included logical object ID1", one instance of 
        "LogicalItemRevisionDomainA" will be returned for the root Item Revision which satifies the input configuration
        context criteria.
        
        The presented properties on the instances of "LogicalItemRevision" and its included logical object:
        "LogicalItemRevisionDomainA" will be returned through Service Data.
        """
        return cls.execute_soa_method(
            method_name='executeBOQueriesWithSortAndContext',
            library='Query',
            service_date='2020_04',
            service_name='SavedQuery',
            params={'inputs': inputs},
            response_cls=SavedQueriesResponse,
        )



from __future__ import annotations

from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from tcsoa.gen.BusinessObjects import ImanFile, Dataset
from typing import Dict, List


@dataclass
class FileDigestInfoResponse(TcBaseObj):
    """
    This structure contains the details about the digest information for the ImanFile objects.
    
    :var fileDigestInfo: A map (ImanFile, list of DigestInfo) of input ImanFile objects to the digest informationa for
    the file.
    :var serviceData: The ServiceData response that can contain the partial errors.
    """
    fileDigestInfo: FileDigestInfoMap = None
    serviceData: ServiceData = None


@dataclass
class FileDigestInfoSet(TcBaseObj):
    """
    This structure contains the digest information available for each of the files in the map.
    
    :var map: A map of ImanFile objects to a list of DigestInfo objects for each digest available for the file.
    """
    map: FileDigestInfoMap = None


@dataclass
class DatasetDigestInfoResponse(TcBaseObj):
    """
    this structure contains the details about the digest information for the ImanFile objects in the Dataset objects.
    
    :var datasetDigestInfo: A list of DatasetDigestInfos objects. It contains the digest information for each file
    referenced by each of the input dataset object.
    :var serviceData: The ServiceData response that can contain the partial errors.
    """
    datasetDigestInfo: DatasetDigestInfoMap = None
    serviceData: ServiceData = None


@dataclass
class DigestInfo(TcBaseObj):
    """
    Structure containing information about an individual digest for an ImanFile object.
    
    :var digestAlgorithm: The digest algorithm used to the compute the digest for this file. This can be one of:
    - None
    - MD5
    - SHA-1
    - SHA-256
    
    
    :var digest: The digest for the file. Message digest algorithms ("digests") are widely used cryptographic hash
    functions.  Digests are utilized in a wide variety of cryptographic applications, and are also commonly used to
    verify data integrity.  FMS uses digest algorithms to produce a hash value of the contents of whole binary files,
    and expresses the hash value in text format as a hexadecimal number. An empty string is returned if no digest was
    available for the file.
    :var certainty: The certainty of the digest stored. The possible values are:
    - 0 - default for unknown or unparsable certainty values.
    - 1 - absolute certainty, a digest generated on the first generation of a file (before an FMS network transport)
    - 2 - high level of certainty, a digest generated as a file is stored in the volume (after an FMS network transport)
    - 3 - medium level of certainty, a digest generated on an existing file in a volume (sometime after the initial
    write to the server)
    - 4 - lowest level of certainty, a digest generated on the fly.
    
    """
    digestAlgorithm: str = ''
    digest: str = ''
    certainty: int = 0


"""
A map of input Dataset objects to a FileDigestInfoSet object, which contains a map of all the files referenced by the Dataset to the list of DigestInfo objects available for that file.
"""
DatasetDigestInfoMap = Dict[Dataset, FileDigestInfoSet]


"""
A map of ImanFile objects to a list of DigestInfo objects for each digest available for the file.
"""
FileDigestInfoMap = Dict[ImanFile, List[DigestInfo]]
